// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js

var handleOpenURL = function(url) {
    window.localStorage.setItem('external_load', url);
};

var RiracleApp = angular
    .module('starter', [
        'ionic',
        'ngAnimate',
        'flash',
        'base64',
        // 'authenticationModule',
        'apiRequesterModule',
        'dbModule',
        'errorModule',
        'pascalprecht.translate',
        'ngCordova',
        'angular-barcode',
        'angular-moment',
        'monospaced.qrcode',
    ])
    .run(function(
        $ionicPlatform,
        $rootScope,
        $ionicModal,
        $ionicLoading,
        $ionicPopup,
        $ionicHistory,
        $state,
        $timeout,
        $moment,
        config,
        DbFactory,
        apiRequesterService,
        errorFactory,
        checkConnectionService,
        headerTitleService
    ) {

        $ionicPlatform.ready(function() {
            document.addEventListener("deviceready", onDeviceReady, false);
            // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
            // for form inputs)
            if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                cordova.plugins.Keyboard.disableScroll(true);
            }

            if( ionic.Platform.isAndroid()) {
              var initialScreenSize = window.innerHeight;
              window.addEventListener("resize", function() {
                if(window.innerHeight < initialScreenSize){
                  document.getElementsByClassName('footer')[0].style.visibility = 'hidden';
                  $('.section').addClass('section--bottom');
                } else {
                  document.getElementsByClassName('footer')[0].style.visibility = 'visible';
                  $('.section').removeClass('section--bottom')
                }
              });
            }
            else {
              var initialScreenSize = screen.height;
              window.addEventListener("resize", function() {
                if(screen.height < initialScreenSize){
                  document.getElementsByClassName('footer')[0].style.visibility = 'hidden';
                  $('.section').addClass('section--bottom');
                } else {
                  document.getElementsByClassName('footer')[0].style.visibility = 'visible';
                  $('.section').removeClass('section--bottom')
                }
              });
            }

            if (window.StatusBar) {
                // org.apache.cordova.statusbar required
                StatusBar.overlaysWebView(false);
                StatusBar.styleDefault();
            }

            document.addEventListener("offline", function(){
                updateStatus(navigator.connection.type, false);
            }, false);

            document.addEventListener("online", function(){
                updateStatus(navigator.connection.type, true);
            }, false);

            if (navigator && navigator.splashscreen) {
                navigator.splashscreen.hide();
            }
        });

        $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/alert-modal.html', {
            scope: $rootScope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $rootScope.modal = modal;
        });

        $rootScope.openModal = function() {
            $rootScope.modal.show();
        }

        $rootScope.closeModal = function() {
            $rootScope.modal.hide();
        };

        $rootScope.$on('$destroy', function() {
            $rootScope.modal.remove();
        });

        $rootScope.$on('modal.hidden', function() {
            // Execute action
        });
          // Execute action on remove modal
        $rootScope.$on('modal.removed', function() {
            // Execute action
        });

        // SHOW VERSION NOTIFICATION MODAL
        $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/version-notification.html', {
            scope: $rootScope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $rootScope.modal = modal;
        });
        // END VERSION NOTIFICATION MODAL

        function setBrightness() {
            // change screen brightness in coupon & mypage screen
            let brightness = window.cordova.plugins.brightness;
            if ($state.current.url != "/coupons/list" && $state.current.url != "/home/mypage") {
                brightness.setBrightness(default_brightness_level);
            } else {
                brightness.setBrightness(1);
            }
        }

        function showNews(state_name) {
            apiRequesterService.checkConnection().then(function(res) {
                apiRequesterService.getNews(1, 1, 1, state_id).then(function(res) {
                    $rootScope.News = res.data.News;

                    if ($rootScope.News.url1 != null && state_name == 'news-detail') {
                        $state.go('news');
                    } else if ($rootScope.News.url1 != null && state_name == 'campaign-detail') {
                        $state.go('campaign-list');
                    } else {
                        headerTitleService.setHeaderName('ニュース詳細');
                        $rootScope.News.start_dt = res.data.News.start_dt.split(' ')[0];
                        $state.go('news-detail', {'id':res.data.News.id});
                    }
                }, function(err) {
                    console.log(err);
                });
            },function(res){
                console.log(err);
            });
        }

        var default_brightness_level = 0.5;

        function onDeviceReady() {
            let brightness = window.cordova.plugins.brightness;
            brightness.getBrightness(function(level) {
              default_brightness_level = level;
              $rootScope.$on('$stateChangeSuccess', function() {
                setBrightness();
              });
            }, function(error) {
              console.log("Error: " + error);
            });
            setTimeout(function() {
                if(typeof check_fcm_loaded==='undefined'){
                    var check_fcm_loaded = setInterval(function(){
                        if(typeof FCMPlugin!=='undefined'){
                            FCMPlugin.onNotification(function(data){
                                if(data.wasTapped){ // if notification was tapped in background
                                    var state_name = null; // for redirection

                                    if (ionic.Platform.isIOS()) {
                                        state_id   = data['gcm.notification.state_id'];
                                        state_name = data['gcm.notification.state_name'];
                                    } else {
                                        state_id   = data['state_id'];
                                        state_name = data['state_name'];
                                    }

                                    // WebView
                                    switch (state_name) {
                                        case 'terms':
                                            $rootScope.parameter = config.webServer + '/usr/terms';
                                            break;

                                        case 'policy':
                                            $rootScope.parameter = config.webServer + '/company/policy';
                                            break;

                                        case 'about-point':
                                            $rootScope.parameter = config.webServer + '/usr/about_point';
                                            break;

                                        case 'company':
                                            $rootScope.parameter = config.webServer + '/company';
                                            break;

                                        case 'contact-us':
                                            $rootScope.parameter = config.webServer + '/usr/inquiry/index';
                                            break;

                                        case 'cancel':
                                            $rootScope.parameter = config.webServer + '/usr/mailfaq';
                                            break;
                                    }

                                    state_name = ( (!$rootScope.logged_in && state_name == 'mypage') ||
                                                   (!$rootScope.logged_in && state_name == 'history') ||
                                                   (!$rootScope.logged_in && state_name == 'orders') ||
                                                   (!$rootScope.logged_in && state_name == 'info')  ||
                                                   (!$rootScope.logged_in && state_name == 'surveys') ) ? 'login' : state_name;

                                    if (typeof state_id !== 'undefined' && state_id != '' && state_id != null) {
                                        showNews(state_name);
                                    } else {
                                        $state.go(state_name);
                                    }

                                }else{ // if notification happened in foreground
                                  //Notification was received in foreground. Maybe the user needs to be notified.
                                }
                            }, function(msg){ // function registered successfuly
                                // alert(msg); // typically msg='OK'
                            }, function(err){
                                console.log(err);
                            });
                            clearInterval(check_fcm_loaded);
                        }
                    }, 1500);
                }
            }, 1500);

            $timeout(function() {
                if(window.localStorage.getItem('external_load') !== null) {
                    var uri = window.localStorage.getItem('external_load');
                    if(uri == 'riracle://confirm') {
                        $state.go('confirm-code');
                    }
                    window.localStorage.removeItem('external_load');
                    $rootScope.visit_home = null;
                }
            },500);

            document.addEventListener("resume", function (event) {
                if(window.localStorage.getItem('external_load') !== null) {
                    var uri = window.localStorage.getItem('external_load');
                    if(uri == 'riracle://confirm') {
                        $state.go('confirm-code');
                    }
                    window.localStorage.removeItem('external_load');
                }
            });

            if(ionic.Platform.isAndroid()) {
                document.addEventListener('resume', onResume, false);
                document.addEventListener("pause", onPause, false);
            } else {
                document.addEventListener('active', onResume, false);
                document.addEventListener("resign", onPause, false);
            }
        }

        function onResume() {
            setBrightness();
        }

        function onPause() {
            let brightness = window.cordova.plugins.brightness;
            brightness.setBrightness(default_brightness_level);
        }


        function updateStatus(connectionType, connectionStatus) {
            checkConnectionService.setConnectionStatus(connectionType, connectionStatus);
        }

        var backURLs = [
            '/confirm',
            '/contact-us',
            '/customers/cancel',
            '/customers/change-address',
            '/customers/change-email',
            '/customers/change-tel',
            '/customers/info',
            '/customers/newpass',
            '/customers/deactivate',
            '/customers/register-change-email',
            '/customers/register-sms-verification',
            '/forgot',
            '/news/detail/:id',
            '/point/about',
            '/pos-customers/terms',
            '/pos-customers/register',
            '/pos-customers/register-confirm',
            '/reviews/add',
            '/shops/city',
            '/shops/list',
            '/shops/pref',
            '/stamp/info',
            '/stamp/shop-list',
            '/static/company',
            '/static/policy',
            '/static/terms',
            '/therapist-ranks/pref',
            '/webview'
        ];

        var headerTitles = {
            'confirm':'会員認証',
            'register':'ログイン情報登録',
            'reg-success':'仮登録完了',
            'login':'ログイン／新規会員登録',
            'login-no-register':'ログイン',
            'forgot':'パスワード再発行',
            'confirm-code':'認証コード',
            'new-passwd':'新規パスワード登録',
            'new-passwd-success':'パスワード登録完了',
            'home':'',
            'info':'会員情報の確認・変更',
            'newpass':'パスワードの変更',
            'change-email':'メールアドレスの変更',
            'deactivate':'退会手続き',
            'orders':'施術履歴',
            'terms':'利用規約',
            'policy':'ポリシー',
            'company':'会社概要',
            'version':'アプリバージョン情報',
            'step1':'',
            'history':'ポイント履歴',
            'change-tel':'電話番号変更',
            'change-address':'住所変更',
            'pref':'都道府県から探す',
            'city':'',
            'list':'',
            'mypage':'マイページ',
            'add':'レビュー',
            'therapist-ranking-pref':'都道府県を選ぶ',
            'coupons':'クーポン',
            'therapist-ranking-list':'指名ランキング',
            'news':'ニュース',
            'news-detail':'ニュース詳細',
            'campaign-list':'キャンペーン情報',
            'campaign-detail':'キャンペーン情報',
            'web-view':'',
            'surveys':'',
            'map':'店舗検索',
            'surveys-questions':'',
            'contact-us':'お問い合わせ',
            'about-point':'ポイントについて',
            'stamp-rally-shop_list' : '店舗一覧',
            'stamp-rally':'スタンプラリー',
            'pos-terms':'メンバーズカード規約',
            'pos-register':'メンバーズカード登録',
            'pos-register-confirm':'メンバーズカード登録',
            'register-activate':'認証コード',
            'register-change-email':'メールアドレス変更',
            'register-success':'会員登録完了'
        };

        // get event "back button tap"
        document.addEventListener("backbutton", onBack, false);

        function onBack() {
            hideSideMenu();
        }

        $rootScope.$on('$stateChangeSuccess', function(event, toState, toParams, fromState, fromParams) {
            hideSideMenu();
            displayBackButton();
            setHeaderTitle();

            // Check server connection on specific states
            if (   toState.name === 'about-point'
                || toState.name === 'terms'
                || toState.name === 'policy'
                || toState.name === 'version'
                || toState.name === 'company'
                || toState.name === 'deactivate'
                || toState.name === 'change-email'
                || toState.name === 'newpass'
                || toState.name === 'change-tel'
                || toState.name === 'contact-us'
            ) {
                let apiRequesterService = angular.element(document.body).injector().get('apiRequesterService');
                apiRequesterService.checkConnection();
            }
        });

        function hideSideMenu() {
            $('.navigation').removeClass('navigation--open');
            $('.header__menu').removeClass('header__menu--close');
            $('span.close-clk').click();
        }

        function displayBackButton() {
            if (backURLs.indexOf($state.current.url) !== -1) {
                $rootScope.showBackButton = true;
                $('.header__menu').addClass('header__menu--back');
            } else {
                $rootScope.showBackButton = false;
                $('.header__menu').removeClass('header__menu--back');
            }
        }
        function setHeaderTitle() {
            if (headerTitles[$state.current.name] != undefined && headerTitles[$state.current.name] != '') {
                headerTitleService.setHeaderName(headerTitles[$state.current.name]);
            }
        }
    })
    .constant('config', {
        server: API_HOST,
        imageServer: API_IMAGE_HOST,
        posImageServer: POS_IMAGE_HOST,
        webServer: WEB_HOST,
        timeout: 10000,
        isIOS: ionic.Platform.isIOS(),
        isAndroid: ionic.Platform.isAndroid(),
        loading: {
            content: 'Loading',
            animation: 'fade-in',
            showBackdrop: true,
            maxWidth: 200,
            showDelay: 0
        },
        language: API_LANGUAGE
    })
.factory('httpRequestInterceptor',['$injector', '$rootScope', '$q',  function ($injector, $rootScope, $q ) {
    function sendRegenerateToken() {
        var apiRequesterService = $injector.get('apiRequesterService');
        apiRequesterService.regenerateAccessToken($rootScope.Auth, $rootScope.Customer).then(function(res) {
            var DbFactory = $injector.get('DbFactory');
            $rootScope.Auth = res.data.AuthCustomer;
            DbFactory.drop('auth_customer').then(function() {
                // save auth to local
                var authData = [$rootScope.Auth];
                DbFactory.insert('auth_customer', authData);
            });
        }).catch(function(err){
        });
    }
    return {
        response: function(res) {
            if (res.status != '-1') {
                var checkConnectionService = $injector.get('checkConnectionService');
                checkConnectionService.setConnectionStatus(true, true);
            }
            return res;
        },
        // Catch ERROR
        responseError: function(res) {
            // if this is true meaning this HTTP request X-Riracle-Token is set
            if (res.status == 401 && res.data.Error.message == 'TokenExpired') {
                sendRegenerateToken();
                return $q.reject($rootScope.before_error);
            } else if(res.status == '-1') { // cause by timeout
                // hide ionic loading
                $injector.get('$ionicLoading').hide();
                $injector.get('checkConnectionService').setConnectionStatus(false, false);
            } else if (res.status == 401 && res.data.Error.message == 'Authentication Failed') { // use by other device
                $injector.get('errorFactory').logoutErrorHandling();
                return $q.resolve(true);
            } else {
                $rootScope.before_error = res;
            }
            return $q.reject(res);
        }
    };
}]).config(function($stateProvider, $urlRouterProvider, $httpProvider, $ionicConfigProvider, $base64, $logProvider,$locationProvider,$translateProvider, config) {
    // avoid to swipe back in iOS
    $ionicConfigProvider.views.swipeBackEnabled(false);

    // add translation tables
    $translateProvider.translations('ja', translationsJA);
    $translateProvider.translations('en', translationsEN);
    $translateProvider.fallbackLanguage('en');
    // $translateProvider.preferredLanguage('en');
    $translateProvider.use(config.language);

    // $httpProvider.defaults.headers.common = {};
    // $httpProvider.defaults.headers.post = {};
    // $httpProvider.defaults.headers.put = {};
    // $httpProvider.defaults.headers.patch = {};
    // Http Interceptors is angularJs function that intercept all http request
    $httpProvider.interceptors.push('httpRequestInterceptor');

    // $httpProvider.defaults.headers.common['Authorization'] = 'Basic ' + $base64.encode('ynsriraku' + ':' + 'map');
    // In riraku_hp.api_auth.name = 'Test' and riraku_hp.api_auth.password = 'Test'
    $httpProvider.defaults.headers.common = { 'X-Riracle-Authorization' : 'UmlyYWNsZTIwMThGWVhadDlxcVZZYzI='}
    $ionicConfigProvider.views.transition('platform');
    $ionicConfigProvider.backButton.text(''); // default is 'Back'
    $ionicConfigProvider.backButton.previousTitleText(false);  // hides the 'Back' text
    $ionicConfigProvider.tabs.position('bottom');
    $logProvider.debugEnabled(true);

    // Ionic uses AngularUI Router which uses the concept of states
    // Learn more here: https://github.com/angular-ui/ui-router
    // Set up the various states which the app can be in.
    // Each state's controller can be found in controllers.js
    $stateProvider

    // Confirm customer code and telephone
    .state('confirm', {
        url: '/confirm',
        templateUrl: 'js/ctrls/customers/register/auth.html',
        controller: 'AuthCtrl'
    })

    // Register customer
    .state('register', {
        url: '/register',
        templateUrl: 'js/ctrls/customers/register/register.html',
        controller: 'RegisterCtrl'
    })

    // Register customer success
    .state('reg-success', {
        url: '/reg-success',
        templateUrl: 'js/ctrls/customers/register/reg-success.html',
        controller: 'RegSuccessCtrl'
    })

    // Login
    .state('login', {
        url: '/login',
        templateUrl: 'js/ctrls/customers/login/login.html',
        controller: 'LoginCtrl',
        cache: false
    })

    // Login no register
    .state('login-no-register', {
        url: '/login-no-register',
        templateUrl: 'js/ctrls/customers/login/login-no-register.html',
        controller: 'LoginCtrl',
        cache: false
    })

    // Forgot password
    .state('forgot', {
        url: '/forgot',
        templateUrl: 'js/ctrls/customers/reissue-passwd/forgot-passwd.html',
        controller: 'ForgotPasswdCtrl'
    })

    // Confirm Code
    .state('confirm-code', {
        url: '/confirm-code',
        templateUrl: 'js/ctrls/customers/reissue-passwd/confirm-code.html',
        controller: 'ConfirmCodeCtrl',
    })

    // New password
    .state('new-passwd', {
        url: '/new-passwd',
        templateUrl: 'js/ctrls/customers/reissue-passwd/new-passwd.html',
        controller: 'NewPasswdCtrl'
    })

    // New password success
    .state('new-passwd-success', {
        url: '/new-passwd-success',
        templateUrl: 'js/ctrls/customers/reissue-passwd/new-passwd-success.html',
        controller: 'NewPasswdSuccessCtrl'
    })

    // setup an abstract state for the tabs directive
    // .state('app', {
    //     url: '/app',
    //     abstract: true,
    //     templateUrl: 'js/ctrls/layouts/navi.html',
    //     controller: 'AppCtrl'
    // })

    // Each tab has its own nav history stack:
    // Pages
    .state('home', {
        url: '/home',
        templateUrl: 'js/ctrls/customers/home/home.html',
        controller: 'HomeCtrl'
        // views: {
        //     'tab-home': {
        //     }
        // }
    })

    // Customer Info page
    .state('info', {
        url: '/customers/info',
        templateUrl: 'js/ctrls/customers/info/info.html',
        controller: 'InfoCtrl'
    })

    // New password page
    .state('newpass', {
        url: '/customers/newpass',
        templateUrl: 'js/ctrls/customers/update-passwd/update-passwd.html',
        controller: 'UpdatePasswdCtrl'
        // views: {
        //     'tab-info': {
        //     }
        // }
    })

    // Change email page
    .state('change-email', {
        url: '/customers/change-email',
        templateUrl: 'js/ctrls/customers/update-email/update-email.html',
        controller: 'UpdateEmailCtrl'
        // views: {
        //     'tab-info': {
        //     }
        // }
    })

    // Deactivate page
    .state('deactivate', {
        url: '/customers/deactivate',
        templateUrl: 'js/ctrls/customers/deactivate/deactivate.html',
        controller: 'DeactivateCtrl'
        // views: {
        //     'tab-info': {
        //     }
        // }
    })

    // Orders List
    .state('orders', {
        url: '/orders/history',
        templateUrl: 'js/ctrls/orders/history.html',
        controller: 'OrderHistoryCtrl'
        // views: {
        //     'tab-orders': {
        //     }
        // }
    })

    // Terms of use page
    .state('terms', {
        url: '/static/terms',
        templateUrl: 'js/ctrls/layouts/common/webview.html',
        // controller: 'TermsCtrl',
    })

     // policy of use page
    .state('policy', {
        url: '/static/policy',
        templateUrl: 'js/ctrls/layouts/common/webview.html',
    })

     // company of use page
    .state('company', {
        url: '/static/company',
        templateUrl: 'js/ctrls/layouts/common/webview.html',
    })

    // App version page
    .state('version', {
        url: '/static/version',
        templateUrl: 'js/ctrls/static/version.html',
        controller: 'VersionCtrl'
        // views: {
        //     'tab-version': {
        //     }
        // }
    })

    // Guidance Step 1
    .state('step1', {
        url: '/static/step1',
        templateUrl: 'js/ctrls/static/step1.html',
        controller: 'Step1Ctrl'
    })

    // POINT HISTORY
    .state('history', {
        url: '/points/history',
        templateUrl: 'js/ctrls/points/history.html',
        controller: 'pointsHistoryCtrl'
    })

    // Change telephone
    .state('change-tel', {
        url: '/customers/change-tel',
        templateUrl: 'js/ctrls/customers/update-tel/update-tel.html',
        controller: 'UpdateTelCtrl'
     })

    // update address page
    .state('change-address', {
        url: '/customers/change-address',
        templateUrl: 'js/ctrls/customers/update-address/update-address.html',
        controller: 'UpdateAddressCtrl'
        // views: {
        //     'tab-info': {
        //     }
        // }
    })

    // shop search pref
    .state('pref', {
        url: '/shops/pref',
        templateUrl: 'js/ctrls/shops/search/pref.html',
     })

    // shop search pref
    .state('city', {
        url: '/shops/city',
        templateUrl: 'js/ctrls/shops/search/city.html',
    })

    // shop search list
    .state('list', {
        url: '/shops/list',
        templateUrl: 'js/ctrls/shops/search/list.html',
     })

    // Customer My Page
    .state('mypage', {
        url: '/home/mypage',
        templateUrl: 'js/ctrls/customers/home/mypage.html',
        controller: 'MyPageCtrl'
    })

    // Add Reviews
    .state('add', {
        url: '/reviews/add',
        templateUrl: 'js/ctrls/reviews/add.html',
        params : { order_id : null}
    })

    // Therapist ranking pref
    .state('therapist-ranking-pref', {
        url: '/therapist-ranks/pref',
        templateUrl: 'js/ctrls/therapist-ranks/pref.html'
    })

    // Add coupons
    .state('coupons', {
      url: '/coupons/list',
      templateUrl: 'js/ctrls/coupons/list.html',
      controller: 'ListCtrl'
    })

     // Therapist ranking list
    .state('therapist-ranking-list', {
        url: '/therapist-ranks/list',
        templateUrl: 'js/ctrls/therapist-ranks/list.html',
    })

    // News Page
    .state('news', {
      url: '/news/list',
      templateUrl: 'js/ctrls/news/list.html'
    })

    // News Page
    .state('news-detail', {
      url: '/news/detail/:id',
      templateUrl: 'js/ctrls/news/detail.html'
    })

    // Campaign list Page
    .state('campaign-list', {
      url: '/campaigns/list',
      templateUrl: 'js/ctrls/campaigns/list.html'
    })

    // Campaign Detail Page
    .state('campaign-detail', {
      url: '/campaigns/detail/:id',
      templateUrl: 'js/ctrls/campaigns/campaign-detail.html',
    })

    // webView Page
    .state('web-view', {
      url: '/webview',
      templateUrl: 'js/ctrls/layouts/common/webview.html',
      controller: 'WebViewCtrl'
    })

    // Surveys Home
    .state('surveys', {
      url: '/surveys/home',
      templateUrl: 'js/ctrls/surveys/home.html',
      controller: 'SurveyHomeCtrl'
    })

    // Map page
    .state('map', {
        url: '/shops/map',
        templateUrl: 'js/ctrls/shops/map.html',
        controller: 'MapCtrl'
    })

    // Surveys Questions
    .state('surveys-questions', {
      url: '/surveys/questions',
      templateUrl: 'js/ctrls/surveys/questions.html',
      controller: 'SurveyQuestionsCtrl'
    })

    // webView Page
    .state('contact-us', {
      url: '/contact-us',
      templateUrl: 'js/ctrls/layouts/common/webview.html',
    })

    // webView Page
    .state('about-point', {
      url: '/point/about',
      templateUrl: 'js/ctrls/layouts/common/webview.html',
    })

    .state('stamp-rally', {
      url: '/stamp/info',
      templateUrl: 'js/ctrls/stamp/info.html',
      controller: 'StampInfoCtrl'
    })

    // stamp rally shop list
    .state('stamp-rally-shop_list', {
      url: '/stamp/shop-list',
      templateUrl: 'js/ctrls/stamp/shop-list.html',
      controller: 'ShopListCtrl'
    })

    // pos registration terms of service
    .state('pos-terms', {
      url: '/pos-customers/terms',
      templateUrl: 'js/ctrls/pos-customers/terms.html',
    })

    // pos registration info input
    .state('pos-register', {
      url: '/pos-customers/register',
      templateUrl: 'js/ctrls/pos-customers/register.html',
    })

    // pos registration confirmation
    .state('pos-register-confirm', {
      url: '/pos-customers/register-confirm',
      templateUrl: 'js/ctrls/pos-customers/register-confirm.html',
      controller: 'PosRegisterConfirmCtrl'
    })

    // registration app account activation
    .state('register-activate', {
      url: '/customers/register-activate',
      templateUrl: 'js/ctrls/customers/register/activate.html',
      controller: 'RegisterActivateCtrl'
    })

    // registration change email
    .state('register-change-email', {
      url: '/customers/register-change-email',
      templateUrl: 'js/ctrls/customers/register/change-email.html',
      controller: 'RegisterChangeEmailCtrl'
    })

    // registration success
    .state('register-success', {
      url: '/customers/register-success',
      templateUrl: 'js/ctrls/customers/register/success.html',
      controller: 'LoginCtrl'
    })

    // registration success
    .state('cancel', {
        url: '/customers/cancel',
        templateUrl: 'js/ctrls/layouts/common/webview.html',
    })

    // sms verification
    .state('register-sms-verification', {
        url: '/customers/register-sms-verification',
        templateUrl: 'js/ctrls/customers/register/sms_verification.html',
        controller: 'registerSMSVerificationCtrl'
    })


    // if none of the above states are matched, use this as the fallback
    if (window.localStorage.getItem('is_fresh_install') != null) {
        $urlRouterProvider.otherwise('/home');
    } else {
        $urlRouterProvider.otherwise('/static/step1');
    }
});
