/*
 * ListCtrl
 *
 * responsible for deactivating customer account
 * view : list.html
 */
(function() {
    'use strict';
    angular
        .module('starter')
        .controller('ListCtrl', ListCtrl);

    function ListCtrl($scope,$rootScope,$ionicLoading, $ionicScrollDelegate, $ionicModal, $timeout, apiRequesterService, config, $state) {
        var customer_id;
        var access_token;
        $scope.$on('$stateChangeSuccess', function() {
             customer_id = $rootScope.Customer != null ? $rootScope.Customer.customer_id : null;
             access_token = $rootScope.Auth != null ? $rootScope.Auth.access_token : null;
             $scope.customer = $rootScope.Customer;
             if ($state.current.url == '/login') { //logging out
                var coupon_param = null;
                apiRequesterService.getCoupons(coupon_param, access_token).then(function(res) {
                   $rootScope.coupon_count = res.data.length;
                }, function(err) {
                    $scope.err = true;
                    $scope.errors = err;
                });
                $scope.customer = null;
             }
        });

        $scope.doRefresh = function() {
            getCoupons();
            getTerms();
        }

        function getCoupons() {
            $ionicLoading.show(config.loading);
            apiRequesterService.getCoupons(customer_id, access_token).then(function(res) {
                if (res.data.length !== 0) {
                    $scope.hasCoupons = true;
                    $scope.before_update_coupons = [];
                    $rootScope.coupon_count = res.data.length;

                    $scope.image_path = config.posImageServer + '/images/coupon_images/'; // staging / production
                    angular.forEach(res.data, function(value, key) {
                        $scope.before_update_coupons.push({
                            id : value.Coupon.id,
                            img : $scope.image_path + value.PosRewardItem.image_file_name,
                            date : value.PosRewardItem.end_date_of_validity,
                            barcode : value.PosRewardItem.id,
                            is_non_member: value.PosRewardCategory.is_available_to_be_used_by_non_member,
                            is_login_user: (value.PosRewardItem.target == 2 || value.PosRewardItem.target == 3) ? 1 : 0,
                            remarks: (value.PosRewardItem.terms_of_use_per_coupon != null && value.PosRewardItem.terms_of_use_per_coupon != '') ? value.PosRewardItem.terms_of_use_per_coupon.split(/\r?\n/g) : null,
                            modified: value.PosRewardItem.modified
                        });
                    });
                    if ($rootScope.coupons == null) {
                        $rootScope.coupons = $scope.before_update_coupons;
                    } else {
                        // if not equal append new
                        if ($scope.before_update_coupons.length != $rootScope.coupons.length) {
                            $scope.hasCoupons = false;
                            $rootScope.coupons = [];
                            $timeout(function(){
                                $rootScope.coupons = $scope.before_update_coupons;
                                $scope.hasCoupons = true;
                            });
                        } else {
                            angular.forEach($rootScope.coupons, function(v,k) {
                                if (v.id != $scope.before_update_coupons[k].id ||
                                    v.modified != $scope.before_update_coupons[k].modified ||
                                    v.is_non_member != $scope.before_update_coupons[k].is_non_member) {
                                    $rootScope.coupons[k] =  $scope.before_update_coupons[k];
                                }
                            });
                            $timeout(function() {

                                var $slick = $('.slide__list');
                                  $slick.slick("unslick");
                                  $slick.slick('destroy');
                                  $slick.on('init', function(event, slick){
                                    $(".slide__item").each(function(){
                                      var slideHeight = this.offsetHeight;

                                      var notesElement = $(this).find('div.notes.notes--coupon');

                                      if(notesElement.length !== 0){
                                        var topPadding = slideHeight * 0.25;

                                        notesElement.css({
                                          "height": slideHeight,
                                          "padding-top": topPadding
                                        });
                                      }
                                    });
                                  });

                                  $slick.slick({
                                    infinite: true,
                                    slidesToShow: 1,
                                    slidesToScroll: 1,
                                    dots: true,
                                    initOnload: true,
                                    data: "coupons",
                                    centerMode: true,
                                    centerPadding: '30px',
                                    prevArrow: $('#js-arrow-prev'),
                                    nextArrow: $('#js-arrow-next'),
                                  });
                            });
                        }
                    }
                } else {
                    $scope.hasCoupons = false;
                }
            }, function(err) {
                $scope.errors = err;
            });
        }

        function getTerms() {
            $rootScope.terms = [];
            apiRequesterService.getCouponTerms().then(function(res) {
                if (res.data.length !== 0) {
                    angular.forEach(res.data, function(value, key) {
                        angular.forEach(value.CouponTerm.terms.split(/\r?\n/g), function(value, key) {
                            $rootScope.terms.push(
                                value
                            );
                        });
                    });
                }
                $ionicLoading.hide();
                $scope.$broadcast('scroll.refreshComplete');
            }, function(err) {
                $scope.errors = err;
                $ionicLoading.hide();
                $scope.$broadcast('scroll.refreshComplete');
            });
        }

        $scope.$on("$ionicView.enter", function( scopes, states ) {
            if ($rootScope.coupons_visited == 1) {
                $scope.hasCoupons = $rootScope.coupon_count > 0;
                apiRequesterService.checkConnection();
            } else {
                $scope.hasCoupons = false;
                $scope.before_update_coupons = null;
                $rootScope.coupons = null;
                $rootScope.terms = [];
                $rootScope.coupons_visited = 1;
                getCoupons();
                getTerms();
            }
            $ionicLoading.hide();
        });
    }
})();

