/*
 * HomeCtrl
 *
 * responsible for displaying home screen
 * view : home.html
 */
(function() {
    'use strict';

    angular
        .module('starter')
        .controller('HomeCtrl', HomeCtrl);

    function HomeCtrl($scope, $log, $rootScope,$moment, $state, $timeout, $http, $ionicLoading, $ionicModal, $ionicPopup, $base64, apiRequesterService, config, DbFactory, errorFactory, $ionicPlatform, checkConnectionService,$ionicHistory, $cordovaInAppBrowser) {

        $scope.order = null;
        $scope.emptyBanners = $rootScope.banner_exist;
        var limit = 4;
        var is_top_in_app = 'app';
        $scope.disable_refresh = false;
        var dragging = false;
        $scope.refreshing = false;
        $rootScope.home_loaded = false;
        var review_flag = null;

        $scope.doRefresh = function() {
            $scope.refreshing = true;
            $scope.order = null;
            // $rootScope.point_modal_shown = null;
            $rootScope.home_loaded = false;

            $rootScope.news_limit = 3;
            apiRequesterService.getNews(1, $rootScope.news_limit).then(function(res){
              $rootScope.news_shop = [];
              appendToArray($rootScope.news_shop, res.data);
            });

            refreshCustomerData();
            getBannersFromAPI();
            $timeout(function(){
                $scope.$broadcast ('updateSlider');
            },2000);
        }

        document.addEventListener("offline", function(){
            updateStatus(navigator.connection.type, false);
        }, false);

        document.addEventListener("online", function(){
            updateStatus(navigator.connection.type, true);
        }, false);

        function updateStatus(connectionType, connectionStatus) {
            checkConnectionService.setConnectionStatus(connectionType, connectionStatus);
        }

        function refreshCustomerData (after_login) {
            $ionicLoading.show(config.loading);
            // Default value of points
            $scope.Customer = {};
            $scope.customer = $rootScope.Customer;

            if (typeof $rootScope.Customer != 'undefined' &&  $rootScope.Customer != null) {
                $rootScope.connected = 'connected';

                apiRequesterService.getPreviousStampRally($rootScope.Auth.customer_id, $rootScope.Auth).then(function(res) {
                    if (res.data.length !== 0 && $rootScope.first_home_visit != true) {
                        $scope.stamp = res.data.PosMstStampRally;
                        DbFactory.tableExist('previous_stamp').then(function() {
                            DbFactory.get('previous_stamp').then(function(data) {
                              if ($scope.stamp.id != data[0].id) {
                                $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/alert-stamp-modal.html', {
                                    scope: $scope,
                                    animation: 'slide-in-up'
                                }).then(function(modal) {
                                    $scope.modal = modal;
                                });
                                DbFactory.drop('previous_stamp').then(function(){
                                    DbFactory.insert('previous_stamp', [$scope.stamp]);
                                })
                              }
                            });
                        }, function() {
                            DbFactory.insert('previous_stamp', [$scope.stamp]);
                            $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/alert-stamp-modal.html', {
                                scope: $scope,
                                animation: 'slide-in-up'
                            }).then(function(modal) {
                                $scope.modal = modal;
                            });
                        });
                    }
                });

                // If go to home after login no need to get customer info again
                if (!after_login) {
                  apiRequesterService.getInfo($rootScope.Customer.customer_id, $rootScope.Auth.access_token).then(function(res) {
                      $rootScope.Customer = "";
                      $rootScope.Customer = res.data.Customer;
                      setBalancePoints();
                      DbFactory.drop('customer').then(function() {
                          // save customer data to local
                          var customerData = [$rootScope.Customer];
                          DbFactory.insert('customer', customerData);
                      });

                      $scope.customer = $rootScope.Customer;
                      $rootScope.logged_in = true;

                  }, function(err) {
                      if (err == 'TokenExpired') {
                          $scope.$broadcast('scroll.refreshComplete');
                          errorFactory.logoutErrorHandling();
                      }
                      resetPoints();
                  });
                } else {
                  setBalancePoints();
                }

                let limit  = 1;
                let offset = 0;
                // If go to home after login no need to get latest order again
                if (!after_login) {
                  apiRequesterService.getOrders($rootScope.Customer.customer_id,$rootScope.Auth, limit, offset,true).then(function(res) {
                      if (res.data.length !== 0) {
                          angular.forEach(res.data, function(value, key){
                              review_flag = typeof value.Order.Review != 'undefined' ?  true : null;
                              $scope.order = {
                                  registerd_business_date: value.Order.registered_business_date,
                                  order_detail_id: value.Order.OrderDetail.id,
                                  customer_id: $rootScope.Customer.customer_id,
                                  product_names : value.Order.product_names,
                                  total_sub_total : value.Order.total_sub_total,
                                  total_amount_of_price : value.Order.total_amount_of_price,
                                  shop_name: value.Order.shop_name,
                                  therapist_name: $scope.mySplit(value.Order.OrderDetail.TherapistNomination.therapist_name, 0),
                                  treated_body_parts: value.Order.TreatedBodyPart ,
                                  treated_body_head: parseInt(value.Order.TreatedBodyPart.strength_of_front_head) + parseInt(value.Order.TreatedBodyPart.strength_of_back_head) + parseInt(value.Order.TreatedBodyPart.strength_of_side_of_head),
                                  treated_body_shoulder: parseInt(value.Order.TreatedBodyPart.strength_of_left_shoulder) + parseInt(value.Order.TreatedBodyPart.strength_of_right_shoulder),
                                  treated_body_arm: parseInt(value.Order.TreatedBodyPart.strength_of_left_arm) + parseInt(value.Order.TreatedBodyPart.strength_of_right_arm),
                                  treated_body_waist: parseInt(value.Order.TreatedBodyPart.strength_of_left_waist) + parseInt(value.Order.TreatedBodyPart.strength_of_right_waist),
                                  treated_body_thigh: parseInt(value.Order.TreatedBodyPart.strength_of_left_thigh) + parseInt(value.Order.TreatedBodyPart.strength_of_right_thigh),
                                  treated_body_calf: parseInt(value.Order.TreatedBodyPart.strength_of_left_calf) + parseInt(value.Order.TreatedBodyPart.strength_of_right_calf),
                                  treated_body_foot: parseInt(value.Order.TreatedBodyPart.strength_of_left_foot) + parseInt(value.Order.TreatedBodyPart.strength_of_right_foot)
                              };
                          });
                          if ($rootScope.visit_home  == 1  && res.data[0].Order.original_order_id == null) { // order must be updated if original_order_id is set
                              $rootScope.visit_home = 2;
                              DbFactory.get('latest_order_detail_id').then(function(data) {
                                  $rootScope.LatestOrder = data[0];
                                  // if LatestOrder is same as in local DB
                                  if ($rootScope.LatestOrder.order_detail_id != $scope.order.order_detail_id) {
                                      // insert if not yet review
                                      if (review_flag == null) {
                                          // remove and add latest_order_detail_id
                                          DbFactory.drop('latest_order_detail_id').then(function() {
                                              var objOrder = [$scope.order];
                                              DbFactory.insert('latest_order_detail_id', objOrder);
                                              DbFactory.get('latest_order_detail_id').then(function(data) {
                                                  $rootScope.LatestOrder = data[0];
                                                  if ($moment(res.data[0].Order.registered_business_date).unix() > $moment().subtract(1, 'Month').unix()) {
                                                      $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/add-review-modal.html', {
                                                          scope: $scope,
                                                          animation: 'slide-in-up'
                                                      }).then(function(modal) {
                                                          $scope.modal_point = modal;
                                                      });
                                                  }
                                              });
                                          });
                                      }
                                  }
                              });
                          }
                      } else {
                          $scope.order = null;
                      }
                  }, function(err) {
                      $scope.order = null;
                      resetPoints();
                  });
              } else {
                $scope.order = $rootScope.order;
              }

              apiRequesterService.getStampRally($rootScope.Customer.customer_id, $rootScope.Auth).then(function(res) {
                if (res.data.length != 0) {
                    $rootScope.StampRally = res.data;
                } else {
                    $rootScope.StampRally = null;
                }
              }, function(err) {
                $rootScope.StampRally = null;
              });
            } else {
                resetPoints();
            }

            $ionicLoading.hide();
            $rootScope.home_loaded = true;

            getCouponsNumber();

            $scope.$broadcast('scroll.refreshComplete');
            $timeout(function(){
                $scope.refreshing = false;
            },1500);
        }

        function getBannersFromAPI() {
            $scope.banners = [];
            var limit = 4;
            var is_top_in_app = 'app';

            apiRequesterService.getBanners(is_top_in_app, limit).then(function(res) {
                if (res.data.length !== 0) {
                    $rootScope.banner_exist = false;
                    angular.forEach(res.data, function(value, key){
                        $scope.banners.push({
                            id : value.Banner.id,
                            image_sp : config.imageServer +'/images/banners/'+  value.Banner.image_sp,
                            news_id : value.Banner.news_id,
                            url : value.Banner.url,
                            is_login_user: value.Banner.is_login_user,
                            is_survey: value.Banner.is_survey,
                        });
                    });
                } else {
                    $rootScope.banner_exist = true;
                }
                $scope.emptyBanners = $rootScope.banner_exist;
            }, function(err) {
                $scope.err = true;
                $scope.errors = err;
                resetPoints();
            });
            $timeout(function(){
                $ionicLoading.hide();
            },3000);
        }

        // return first word from name
        $scope.mySplit = function(string, nb) {
            var array = string.split('　');
            return array[nb];
        }

        $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/alert-modal.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
        });

        $scope.openModal = function() {
            $scope.modal.show();
        }

        $scope.closeModal = function() {
            $scope.modal.hide();
        };

        $scope.$on('$destroy', function() {
            $scope.modal.remove();
        });

        $rootScope.news_limit = 3;

        // get customer detail info and add to rootscope
        $scope.$on('$stateChangeSuccess', function($scope, toState, toParams, fromState) {
            if ($rootScope.first_home_visit != null) {
                if ($rootScope.visit_home == null) {
                    $rootScope.visit_home = 1;
                    $rootScope.point_modal_shown = null;
                    $scope.refreshing = false;
                    // execute this only once
                    DbFactory.tableExist('customer').then(function() {
                        DbFactory.get('customer').then(function(data) {
                            $rootScope.Customer = data[0];
                            $scope.customer = $rootScope.Customer;
                            DbFactory.tableExist('auth_customer').then(function() {
                                DbFactory.get('auth_customer').then(function(data) {
                                    $rootScope.Auth = data[0];
                                    apiRequesterService.setLastLoginApp($rootScope.Customer.customer_id,$rootScope.Auth.access_token).then(function(res) {
                                        // If after log in, set param to true so no need to get some data
                                        if (fromState.name == 'login' && $rootScope.logged_in) {
                                            refreshCustomerData(true);
                                        } else {
                                            refreshCustomerData();
                                        }
                                        $rootScope.home_loaded = true;
                                    }, function(err) {
                                        $scope.err = false;
                                        $scope.errors = undefined;
                                    });
                                });
                            });
                        });
                    });
                    // prevent calling this refreshCustomerData twice in case customer is iexist
                    if (typeof $rootScope.Customer == 'undefined' || $rootScope.Customer == null) {
                        //check if there are banners
                        apiRequesterService.getBanners(is_top_in_app, limit).then(function(res) {
                            if (res.data.length !== 0) {
                                $rootScope.banner_exist = false;
                                $scope.emptyBanners = false;

                            } else {
                                $rootScope.banner_exist = true;
                                $scope.emptyBanners = true;
                            }
                            refreshCustomerData();
                        }, function(err) {
                            $scope.err = true;
                            $scope.errors = err;
                        });
                    }
                    $rootScope.news_limit = 3;
                    apiRequesterService.getNews(1, $rootScope.news_limit).then(function(res){
                      $rootScope.news_shop = [];
                      appendToArray($rootScope.news_shop, res.data);
                    });
                } else {
                    if ($state.current.name == 'home') {
                        apiRequesterService.checkConnection();
                    }
                }

                if (typeof $rootScope.Customer == 'undefined' || $rootScope.Customer == null) {
                    resetPoints();
                } else {
                    setBalancePoints();
                }

                if ($state.current.name == 'home' &&
                    (typeof $rootScope.Customer != 'undefined' && $rootScope.Customer != null && fromState.name != 'login'
                    || (typeof $rootScope.Customer == 'undefined' || $rootScope.Customer == null))) {
                    apiRequesterService.checkConnection();
                }
            }
        });

        // About Point (Web View)
        $scope.aboutPoint = function() {
            if(config.isIOS) {
                var ref = cordova.InAppBrowser.open(config.webServer + '/usr/about_point', '_blank', 'location=no');
                ref.addEventListener('loadstart', function(event) {
                    if (window.cordova && window.cordova.plugins.Keyboard) {
                        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
                    }
                });

                ref.addEventListener('exit', function(event) {
                    if (window.cordova && window.cordova.plugins.Keyboard) {
                        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                    }
                });
            } else {
                $rootScope.parameter = config.webServer + '/usr/about_point';
                $rootScope.title = 'ポイントについて';
                $state.go('about-point');
            }
        }

        // get coupon count
        function getCouponsNumber() {
            var coupon_param = $rootScope.Customer == null ? null : $rootScope.Customer.customer_id;
            var access_token = $rootScope.Auth == null ? null : $rootScope.Auth.access_token;
            apiRequesterService.getCoupons(coupon_param, access_token).then(function(res) {
                $rootScope.coupon_count = res.data.length;
                $rootScope.coupons_visited = false;
            }, function(err) {
                $scope.errors = err;
            });
        }

        $scope.$on( "$ionicView.loaded", function( scopes, states ) {
            if ($rootScope.first_home_visit == null) {
                $rootScope.coupon_count = 0;
                $scope.banners = [];
                $rootScope.news_shop = [];

                $rootScope.point_modal_shown = null;
                $scope.refreshing = false;
                DbFactory.tableExist('customer').then(function() {
                    DbFactory.get('customer').then(function(data) {
                        $rootScope.Customer = data[0];
                        $scope.customer = $rootScope.Customer;
                        DbFactory.tableExist('auth_customer').then(function() {
                            DbFactory.get('auth_customer').then(function(data) {
                                $rootScope.Auth = data[0];
                                getCustomerHome();
                            });
                        });
                    });
                }, function(){
                    getCustomerHome();
                });
            } else {
                getBannersFromAPI();
            }
        });

        function getCustomerHome() {
            $ionicLoading.show();
            let customer_id = typeof $rootScope.Customer != 'undefined' ? $rootScope.Customer.customer_id : null;
            let access_token = typeof $rootScope.Auth != 'undefined' ? $rootScope.Auth.access_token : null;
            apiRequesterService.getCustomerHome(customer_id, access_token).then(function(res) {
                if (res.data.Banners.length !== 0) {
                    angular.forEach(res.data.Banners, function(value, key){
                        $scope.banners.push({
                            id : value.Banner.id,
                            image_sp : config.imageServer +'/images/banners/'+  value.Banner.image_sp,
                            news_id : value.Banner.news_id,
                            url : value.Banner.url,
                            is_login_user: value.Banner.is_login_user,
                            is_survey: value.Banner.is_survey,
                        });
                        $rootScope.banner_exist = false;
                        $scope.emptyBanners = false;
                    });
                } else {
                    $rootScope.banner_exist = true;
                    $scope.emptyBanners = true;
                }

                if (res.data.News.length !== 0) {
                    appendToArray($rootScope.news_shop, res.data.News);
                }

                if (typeof res.data.PreviousStampRally != 'undefined' && res.data.PreviousStampRally.length !== 0) {
                    $scope.stamp = res.data.PreviousStampRally;
                    DbFactory.tableExist('previous_stamp').then(function() {
                        DbFactory.get('previous_stamp').then(function(data) {
                          if ($scope.stamp.id != data[0].id) {
                            $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/alert-stamp-modal.html', {
                                scope: $scope,
                                animation: 'slide-in-up'
                            }).then(function(modal) {
                                $scope.modal = modal;
                            });
                            DbFactory.drop('previous_stamp').then(function(){
                                DbFactory.insert('previous_stamp', [$scope.stamp]);
                            })
                          }
                        });
                    }, function() {
                        DbFactory.insert('previous_stamp', [$scope.stamp]);
                        $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/alert-stamp-modal.html', {
                            scope: $scope,
                            animation: 'slide-in-up'
                        }).then(function(modal) {
                            $scope.modal = modal;
                        });
                    });
                }

                if (typeof res.data.Customer != 'undefined' && res.data.Customer.length !== 0) {
                    $rootScope.Customer = "";
                    $rootScope.Customer = res.data.Customer.Customer;
                    setBalancePoints();
                    DbFactory.drop('customer').then(function() {
                        // save customer data to local
                        var customerData = [$rootScope.Customer];
                        DbFactory.insert('customer', customerData);
                    });

                    $scope.customer = $rootScope.Customer;
                    $rootScope.logged_in = true;

                } else {
                    resetPoints();
                }

                if (typeof res.data.Order != 'undefined' && res.data.Order.length !== 0) {
                    angular.forEach(res.data.Order, function(value, key){
                        review_flag = typeof value.Order.Review != 'undefined' ?  true : null;
                        $scope.order = {
                            registerd_business_date: value.Order.registered_business_date,
                            order_detail_id: value.Order.OrderDetail.id,
                            customer_id: $rootScope.Customer.customer_id,
                            product_names : value.Order.product_names,
                            total_sub_total : value.Order.total_sub_total,
                            total_amount_of_price : value.Order.total_amount_of_price,
                            shop_name: value.Order.shop_name,
                            therapist_name: $scope.mySplit(value.Order.OrderDetail.TherapistNomination.therapist_name, 0),
                            treated_body_parts: value.Order.TreatedBodyPart ,
                            treated_body_head: parseInt(value.Order.TreatedBodyPart.strength_of_front_head) + parseInt(value.Order.TreatedBodyPart.strength_of_back_head) + parseInt(value.Order.TreatedBodyPart.strength_of_side_of_head),
                            treated_body_shoulder: parseInt(value.Order.TreatedBodyPart.strength_of_left_shoulder) + parseInt(value.Order.TreatedBodyPart.strength_of_right_shoulder),
                            treated_body_arm: parseInt(value.Order.TreatedBodyPart.strength_of_left_arm) + parseInt(value.Order.TreatedBodyPart.strength_of_right_arm),
                            treated_body_waist: parseInt(value.Order.TreatedBodyPart.strength_of_left_waist) + parseInt(value.Order.TreatedBodyPart.strength_of_right_waist),
                            treated_body_thigh: parseInt(value.Order.TreatedBodyPart.strength_of_left_thigh) + parseInt(value.Order.TreatedBodyPart.strength_of_right_thigh),
                            treated_body_calf: parseInt(value.Order.TreatedBodyPart.strength_of_left_calf) + parseInt(value.Order.TreatedBodyPart.strength_of_right_calf),
                            treated_body_foot: parseInt(value.Order.TreatedBodyPart.strength_of_left_foot) + parseInt(value.Order.TreatedBodyPart.strength_of_right_foot)
                        };
                    });
                    if (res.data.Order[0].Order.original_order_id == null) { // order must be updated if original_order_id is set
                        $rootScope.visit_home = 2;
                        DbFactory.get('latest_order_detail_id').then(function(data) {
                            $rootScope.LatestOrder = data[0];
                            // if LatestOrder is same as in local DB
                            if ($rootScope.LatestOrder.order_detail_id != $scope.order.order_detail_id) {
                                // insert if not yet review
                                if (review_flag == null) {
                                    // remove and add latest_order_detail_id
                                    DbFactory.drop('latest_order_detail_id').then(function() {
                                        var objOrder = [$scope.order];
                                        DbFactory.insert('latest_order_detail_id', objOrder);
                                        DbFactory.get('latest_order_detail_id').then(function(data) {
                                            $rootScope.LatestOrder = data[0];
                                            if ($moment(res.data.Order[0].Order.registered_business_date).unix() > $moment().subtract(1, 'Month').unix()) {
                                                $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/add-review-modal.html', {
                                                    scope: $scope,
                                                    animation: 'slide-in-up'
                                                }).then(function(modal) {
                                                    $scope.modal_point = modal;
                                                });
                                            }
                                        });
                                    });
                                }
                            }
                        });
                    }
                } else {
                    $scope.order = null;
                }

                if (typeof res.data.StampRally != 'undefined' && res.data.StampRally.length !== 0) {
                    $rootScope.StampRally = res.data.StampRally;
                } else {
                    $rootScope.StampRally = null;
                }


                $rootScope.coupon_count = res.data.Coupons.length;

                $rootScope.home_loaded = true;
                $rootScope.visit_home = 1;
                $rootScope.first_home_visit = true;
            }, function(err) {
                if (err == 'TokenExpired') {
                    $scope.$broadcast('scroll.refreshComplete');
                    errorFactory.logoutErrorHandling();
                }
                $scope.order = null;
                $rootScope.StampRally = null;
                resetPoints();
            }).finally(function() {
                $timeout(function(){
                    $ionicLoading.hide();
                },3000);
            });
        }

        function setBalancePoints() {
            $scope.Customer = $rootScope.Customer;
            $scope.Customer.total_points = parseInt($scope.Customer.current_charge_point) + parseInt($scope.Customer.current_bonus_point);
        }
        // reset points if no internet
        function resetPoints() {
            $scope.Customer = {};
            $scope.Customer.total_points = 0;
            $scope.Customer.current_charge_point = 0;
            $scope.Customer.current_bonus_point = 0;
        }

        $scope.goMyPage = function(){
            if (typeof $rootScope.Customer != 'undefined' && $rootScope.Customer != null) {
              $ionicHistory.nextViewOptions({
                  disableAnimate: true
              });
              $state.go('mypage');
            } else {
                $ionicHistory.nextViewOptions({
                    disableAnimate: true
                });
                $state.go('login');
            }
        }

        function appendToArray(news_array, res_data){
          if (news_array.length == 0) {
                angular.forEach(res_data, function(value, key){
                    news_array[value.News.id] = {
                        id : value.News.id,
                        start_dt : value.News.start_dt,
                        title : value.News.title,
                        free_space_pc : value.News.free_space_pc,
                        url : value.News.url1,
                        created : value.News.created
                    };
                });
                formatDates(news_array);
            }
        };
        function formatDates(news_array) {
            angular.forEach(news_array, function(value,key){
                value.start_dt = value.start_dt.split(' ')[0];
            });
        }
    }
})();
