/*
 * RegisterActivateCtrl
 *
 * responsible for registration app account activation
 * view : activate.html
 */
(function () {
    'use strict';

    angular
        .module('starter')
        .controller('RegisterActivateCtrl', RegisterActivateCtrl);

    function RegisterActivateCtrl($scope, $ionicScrollDelegate, $rootScope, $state, $ionicLoading, $ionicPopup, apiRequesterService, config) {
        $scope.form = {
            is_invalid: false,
            members_card_no: '',
            id: '',
            activation_error: '',
            activation_key: ''
        }

        let checkRootScope = function () {
            if (typeof $rootScope.pos_customer !== 'undefined') {
                if (typeof $rootScope.pos_customer.PosCustomer !== 'undefined') {
                    $scope.form.members_card_no = $rootScope.pos_customer.PosCustomer.members_card_no;
                }
                if (typeof $rootScope.pos_customer.Customer !== 'undefined') {
                    $scope.form.id = $rootScope.pos_customer.Customer.id;
                }
            }
        }

        $rootScope.$on('$stateChangeStart', function () {
            $scope.form.is_invalid = false;
            $scope.form.activation_key = '';
            $ionicScrollDelegate.scrollTop();
            checkRootScope();
        })

        $scope.init = function () {
            checkRootScope();
        }

        $scope.activate = function () {
            $ionicLoading.show(config.loading);
            apiRequesterService
                .checkConnection()
                .then(
                    function () {
                        if ($scope.isSmsBased) {
                            apiRequesterService
                                .confirmSmsVerification($scope.form.id, $scope.form.activation_key)
                                .then(
                                    function () {
                                        activationSuccessHandler();
                                    },
                                    function (error) {
                                        activationErrorHandler(error);
                                    }
                                );
                        } else {
                            apiRequesterService
                                .posCustomerActivate($scope.form)
                                .then(
                                    function () {
                                        activationSuccessHandler();
                                    },
                                    function (error) {
                                        activationErrorHandler(error);
                                    }
                                );
                        }
                    },
                    function () {
                        $ionicLoading.hide();
                        $rootScope.connected = 'noConnection';
                    }
                )
        }

        function activationSuccessHandler() {
            $ionicLoading.hide();
            $scope.form.id = '';
            $scope.form.activation_error = '';
            $scope.members_card_no = '';
            $scope.form.is_invalid = false;
            $rootScope.is_customer_registration_page = false;
            $state.go('register-success');
        }

        function activationErrorHandler(error) {
            $ionicLoading.hide();
            $ionicScrollDelegate.scrollTop();
            $scope.form.activation_error = error.Customer.customer_id || '';
            $scope.form.is_invalid = true;
        }

        $scope.openWebsite = function () {
            $ionicPopup.confirm({
                title: 'ブラウザで開く',
                template: 'このページはアプリでは開けないため、ブラウザが起動しますがよろしいですか？',
                buttons: [{
                    text: 'いいえ',
                    type: 'button-disable'
                }, {
                    text: 'はい',
                    type: 'button-positive',
                    onTap: function () {
                        window.open(config.webServer + '/usr/inquiry/index', '_system', 'location=yes')
                    }
                }]
            })
        }

        $scope.cancel = function () {
            if (config.isIOS) {
                var ref = cordova.InAppBrowser.open(config.webServer + '/usr/mailfaq', '_blank', 'location=no');
                ref.addEventListener('loadstart', function () {
                    if (window.cordova && window.cordova.plugins.Keyboard) {
                        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
                    }
                });

                ref.addEventListener('exit', function () {
                    if (window.cordova && window.cordova.plugins.Keyboard) {
                        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                    }
                });
            } else {
                $rootScope.parameter = config.webServer + '/usr/mailfaq';
                $state.go('cancel');
            }
        }

        $scope.$on('$ionicView.enter', function () {
            let isSmsBased;
            if (typeof $rootScope.isSmsBased === 'undefined') {
                isSmsBased = false;
            } else {
                isSmsBased = $rootScope.isSmsBased;
            }

            if (isSmsBased) {
                $scope.isSmsBased = true;
            } else {
                $scope.isSmsBased = false;
            }
        });
    }
})();
