/*
 * registerSMSVerificationCtrl
 *
 * responsible for registration info success
 * view : sms_verification.html
 */
(function () {
    'use strict';

    angular
        .module('starter')
        .controller('registerSMSVerificationCtrl', registerSMSVerificationCtrl);

    function registerSMSVerificationCtrl($scope, $rootScope, $ionicLoading, $ionicPopup, $state, apiRequesterService, config) {

        $scope.Customer = {
            id: $rootScope.pos_customer.Customer.id,
            phone_no: ''
        }

        $scope.openConfirmation = function () {
            $ionicLoading.show();
            apiRequesterService
                .requestSmsVerification($scope.Customer.id, $scope.Customer.phone_no, true)
                .then(
                    function () {
                        handleRequestSmsVerificationSuccess()
                    },
                    function (errors) {
                        handleRequestSmsVerificationError(errors)
                    }
                )
                .finally(
                    function () {
                        $ionicLoading.hide();
                    }
                )
        }

        $scope.openWebsite = function () {
            $ionicPopup.confirm({
                title: 'ブラウザで開く',
                template: 'このページはアプリでは開けないため、ブラウザが起動しますがよろしいですか？',
                buttons: [
                    {
                        text: 'いいえ',
                        type: 'button-disable'
                    },
                    {
                        text: 'はい',
                        type: 'button-positive',
                        onTap: function () {
                            window.open(config.webServer + '/usr/inquiry/index', '_system', 'location=yes')
                        }
                    }
                ]
            });
        }

        function handleRequestSmsVerificationSuccess() {
            $scope.errors = null;
            $ionicPopup.confirm({
                title: $scope.Customer.phone_no,
                template: 'こちらの電話番号へSMS（ショートメッセージ）にて認証コードを送信いたします。<br>送信しますか？',
                cssClass: 'sms-verification',
                buttons: [
                    {
                        text: 'キャンセル',
                        type: 'button-disable'
                    },
                    {
                        text: '送信する',
                        type: 'button-positive',
                        onTap: function () {
                            $ionicLoading.show();
                            apiRequesterService
                                .requestSmsVerification($scope.Customer.id, $scope.Customer.phone_no, false)
                                .then(
                                    function () {
                                        $rootScope.isSmsBased = true;
                                        $state.go('register-activate');
                                    },
                                    function (errors) {
                                        handleRequestSmsVerificationError(errors)
                                    }
                                )
                                .finally(
                                    function () {
                                        $ionicLoading.hide();
                                    }
                                )
                        }
                    }
                ]
            });
        }

        function handleRequestSmsVerificationError(errors) {
            if (errors.hasOwnProperty('Customer')) {
                $scope.errors = errors;
            } else {
                $scope.errors = null;
                $ionicPopup.alert({
                    title: 'エラー',
                    template: 'SMSの送信に失敗しました。入力いただいた電話番号をご確認ください。',
                    buttons: [
                        {
                            text: '閉じる',
                            type: 'button--cancel'
                        }
                    ]
                });
            }
        }
    }
})();
