/*
 * NewsList
 *
 * responsible for updating new password
 * view : list.html
 */
(function() {
    'use strict';

    angular
        .module('starter')
        .controller('NewsList', NewsList);

    function NewsList($scope, $rootScope, $state, $timeout, apiRequesterService) {
        $scope.refreshing = false;
        $rootScope.news_limit = 5;
        $rootScope.noMoreNewsAvailable = false;
        $rootScope.news_shop = $rootScope.news_shop == null ? [] : $rootScope.news_shop;
        $rootScope.news_media = $rootScope.news_media == null ? [] : $rootScope.news_media;
        $rootScope.news_news = $rootScope.news_news == null ? [] : $rootScope.news_news;
        $rootScope.news_page = 0;
        $rootScope.news_offset = 0;

        $scope.doRefresh = function() {
            $scope.refreshing = true;
            getNews($rootScope.current_news_tab);
        }

        $scope.loadMore = function(news_type) {
            if (!$rootScope.noMoreNewsAvailable) {
                getNews(news_type,true);
            }
        }

        function getNews(news_type, options) {
            let global_limit = $rootScope.news_limit ;
            let global_offset = $rootScope.news_offset ;
            if (options == null) {
              global_limit = $rootScope.news_limit;
              global_offset = 0;
            }
            if ($rootScope.current_news_tab == news_type) {
                apiRequesterService.checkConnection().then(function(res){
                    apiRequesterService.getNews(news_type, global_limit, global_offset).then(function(res){
                        if (res.data.length!==0) {
                            if (news_type == 1) {
                                appendToArray($rootScope.news_shop, res.data);
                            } else if (news_type == 2) {
                                appendToArray($rootScope.news_media, res.data)
                            } else {
                                appendToArray($rootScope.news_news, res.data)
                            }
                            if (options != null) {
                                $rootScope.news_offset += $rootScope.news_limit;
                            }

                            $rootScope.noMoreNewsAvailable = false;
                        } else {
                            $rootScope.noMoreNewsAvailable = true;
                        }
                    },function(err){
                        $rootScope.noMoreNewsAvailable = true;
                    }).finally(function() {
                        // Stop the loading from spinning
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                        $scope.$broadcast('scroll.refreshComplete');
                        $timeout(function(){
                            $scope.refreshing = false;
                        },1500);
                    });
                },function(err){
                    $scope.err = false;
                    $scope.errors = undefined;
                    $rootScope.connected = 'noConnection';
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                });
            }
        }

        function appendToArray(news_array, res_data){
            angular.forEach(res_data, function(value, key){
                news_array[value.News.id] = {
                   id : value.News.id,
                   start_dt : value.News.start_dt,
                   title : value.News.title,
                   free_space_pc : value.News.free_space_pc,
                   url : value.News.url1,
                   created : value.News.created
                };
            });
            formatDates(news_array);
        };

        function formatDates(news_array) {
            angular.forEach(news_array, function(value,key){
                value.start_dt = value.start_dt.split(' ')[0];
            });
        }

        $scope.goToDetail = function(news){
            $rootScope.News = news;
            $state.go('news-detail', {'id':news.id});
        }

    }

})();
