/*
 * OrderHistoryCtrl
 *
 * responsible for displaying order history email screen
 * view : order-history.html
 */
(function() {
    'use strict';

    angular
        .module('starter')
        .controller('OrderHistoryCtrl', OrderHistoryCtrl);

    function OrderHistoryCtrl($scope, $log,$ionicLoading, $rootScope, $state, $q, config, apiRequesterService, errorFactory, $timeout, $moment, $ionicPopup) {
        var limit  = 5;
        var offset = 0;
        $scope.show_now = false;
        $scope.noMoreItemsAvailable = false;
        $scope.items = [];

        $scope.doRefresh = function() {
            getOrders();
        }
        $scope.loadMore = function() {
            if (!$scope.noMoreItemsAvailable) {
                getOrders(true);
            }
        }
        var requesting = false;
        var counter = 0;
        function getOrders(options) {
            if (!requesting) {
                let global_limit = limit ;
                let global_offset = offset ;
                if (options == null) {
                    global_limit = offset;
                    global_offset = 0;
                    $rootScope.OrderItems = [];
                    $scope.items = [];
                    $scope.noOrder = true;
                    counter = 0;
                }
                requesting = true;
                apiRequesterService.checkConnection().then(function(res) {
                    apiRequesterService.getOrders($rootScope.Customer.customer_id,$rootScope.Auth, global_limit, global_offset, true).then(function(res) {
                        if (res.data.length !== 0) {
                            $scope.noOrder = false;
                            angular.forEach(res.data, function(value, key){
                                $rootScope.OrderItems[counter] = {
                                    id: value.Order.OrderDetail.id,
                                    order_id: value.Order.id,
                                    registered_business_date: value.Order.registered_business_date,
                                    registered_business_date_unix: $moment(value.Order.registered_business_date).unix(),
                                    product_names : value.Order.product_names,
                                    total_sub_total : value.Order.total_sub_total,
                                    total_amount_of_price : value.Order.total_amount_of_price,
                                    is_available_to_book_online: value.Order.is_available_to_book_online,
                                    url_type: value.Order.url_type,
                                    url_for_booking_online: value.Order.url_for_booking_online_sp,
                                    shop_id: value.Order.no,
                                    shop_name: value.Order.shop_name,
                                    shop_telephone_no: value.Order.shop_telephone_no,
                                    therapist_name: $scope.mySplit(value.Order.OrderDetail.TherapistNomination.therapist_name, 0),
                                    treated_body_parts: value.Order.TreatedBodyPart.id ,
                                    treated_body_head: parseInt(value.Order.TreatedBodyPart.strength_of_front_head) + parseInt(value.Order.TreatedBodyPart.strength_of_back_head) + parseInt(value.Order.TreatedBodyPart.strength_of_side_of_head),
                                    treated_body_shoulder: parseInt(value.Order.TreatedBodyPart.strength_of_left_shoulder) + parseInt(value.Order.TreatedBodyPart.strength_of_right_shoulder),
                                    treated_body_arm: parseInt(value.Order.TreatedBodyPart.strength_of_left_arm) + parseInt(value.Order.TreatedBodyPart.strength_of_right_arm),
                                    treated_body_waist: parseInt(value.Order.TreatedBodyPart.strength_of_left_waist) + parseInt(value.Order.TreatedBodyPart.strength_of_right_waist),
                                    treated_body_thigh: parseInt(value.Order.TreatedBodyPart.strength_of_left_thigh) + parseInt(value.Order.TreatedBodyPart.strength_of_right_thigh),
                                    treated_body_calf: parseInt(value.Order.TreatedBodyPart.strength_of_left_calf) + parseInt(value.Order.TreatedBodyPart.strength_of_right_calf),
                                    treated_body_foot: parseInt(value.Order.TreatedBodyPart.strength_of_left_foot) + parseInt(value.Order.TreatedBodyPart.strength_of_right_foot),
                                    stars: typeof value.Order.Review != 'undefined' ?  value.Order.Review.rank : null
                                };
                                counter++;
                            });
                            $scope.show_now = true;
                            $scope.items = $rootScope.OrderItems;
                            $scope.noMoreItemsAvailable = false;
                        } else  {
                            if (offset == 0) {
                                $scope.noOrder = true;
                            }
                            $scope.noMoreItemsAvailable = true;
                        }
                        $ionicLoading.hide();
                    }, function(err) {
                        $scope.noMoreItemsAvailable = true;
                        $ionicLoading.hide();
                    }).finally(function() {
                        if (options != null) {
                            offset+=limit;
                        }
                        requesting = false;
                        // Stop the loading from spinning
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                        $scope.$broadcast('scroll.refreshComplete');
                        $ionicLoading.hide();
                    });
                },function(res){
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    $scope.noMoreItemsAvailable = true;
                    $rootScope.connected = 'noConnection';
                    $ionicLoading.hide();
                });
            }
        };
        $scope.addReview = function(e) {
            $rootScope.order_select = {order_id:e.order_id,order_detail_id:e.id}
            $state.go('add');
        }
        // return first word from name
        $scope.mySplit = function(string, nb) {
            var array = string.split('　');
            return array[nb];
        }

        // get customer detail info and add to rootscope
        $scope.$on('$stateChangeSuccess', function(event, toState) {
            if (toState.name === 'orders') {
                $scope.one_month_ago = $moment().subtract(1, 'Month').unix();
                if ($rootScope.OrderItems == null) {
                    $rootScope.OrderItems = [];
                } else {
                    $scope.show_now = true
                }
                $scope.items = $rootScope.OrderItems;
                // check if access token is still ok
                apiRequesterService.checkToken($rootScope.Customer.customer_id,$rootScope.Auth.access_token).then(function(res) {
                });
                $scope.noOrder = $scope.items.length == 0 ? true : false;
            }
        });

        getOrders(true);

        $scope.openWebView = function(order) {
          var access_token = $rootScope.Auth.access_token;
          var url = '/';
          if (order.url_type == 1) {
            url = config.webServer + '/usr/reservations/login_from_app?from_shop=' + order.shop_id;
            if (access_token != null) {
              url += '&token=' + access_token;
            }
          }
          if (order.url_type == 2) {
            url = order.url_for_booking_online;
          }
          var confirmPopup = $ionicPopup.confirm({
              title: 'ブラウザで開く',
              template: 'このページはアプリでは開けないため、ブラウザが起動しますがよろしいですか？',

              buttons: [{
                  text: 'いいえ',
                  type: 'button-disable'
              },{
                  text: 'はい',
                  type: 'button-positive',
                  onTap: function(e) {
                      window.open(url, '_system', 'location=yes')
                  }
              }
              ]
          });
        }
    }
})();