/*
 * pointsHistoryCtrl
 *
 * responsible for displaying order history email screen
 * view : order-history.html
 */
(function() {
    'use strict';

    angular
        .module('starter')
        .controller('pointsHistoryCtrl', pointsHistoryCtrl);

    function pointsHistoryCtrl($scope, $rootScope, apiRequesterService, $state, config, $ionicLoading, DbFactory) {
        var offset = 0;
        var limit = 10;
        var requesting = false;

        $scope.Customer = $rootScope.Customer;
        $scope.Customer.total_points = parseInt($scope.Customer.current_charge_point) + parseInt($scope.Customer.current_bonus_point);
        $scope.items = [];
        $scope.noMoreItemsAvailable = false;
        $scope.show_now = false;

        // Handle pull down event
        $scope.doRefresh = function () {
            getPoints(false);
        }

        // Handle loading of additional records
        $rootScope.loadMore = function () {
            if (!$scope.noMoreItemsAvailable) {
                getPoints(true);
            }
        }

        /**
         * Get customer points
         *
         * @param {Boolean} append True if append to previous results. False if not.
         */
        function getPoints(append) {
            if (requesting) {
                return;
            }
            requesting = true;

            if (!append) {
                offset = 0;
                $scope.items = [];
            }

            // Get customer info
            apiRequesterService.getInfo($rootScope.Auth.customer_id, $rootScope.Auth.access_token)
                .then(function (res) {
                    $rootScope.Customer = res.data.Customer;
                    DbFactory.drop('customer').then(function() {
                        // save customer data to local
                        var customerData = [$rootScope.Customer];
                        DbFactory.insert('customer', customerData);
                    });
                    $scope.Customer = $rootScope.Customer;
                    $scope.Customer.total_points = parseInt($scope.Customer.current_charge_point) + parseInt($scope.Customer.current_bonus_point);
                });

            // Load point history
            apiRequesterService.selectPoint($rootScope.Customer.customer_id, $rootScope.Auth.access_token, limit, offset)
                .then(
                    function(res) {
                        if (res.data.length > 0) {
                            angular.forEach(res.data, function(value) {
                                $scope.items.push({
                                    id : value.PosPoint.id,
                                    registered_date: value.PosPoint.registered_date,
                                    point_status_name: value.PosPoint.point_status_name,
                                    history_point: value.PosPoint.history_point,
                                    point_status: value.PosPoint.point_status
                                });
                            });
                            offset += res.data.length;
                            $scope.show_now = true;
                        }
                        $scope.noMoreItemsAvailable = (res.data.length < limit);
                    },
                    function () {
                        $scope.noMoreItemsAvailable = true;
                    }
                )
                .finally(function () {
                    requesting = false;
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    $scope.$broadcast('scroll.refreshComplete');
                    $ionicLoading.hide();
                });
        }

        // About Point (Web View)
        $scope.aboutPoint = function () {
            if (config.isIOS) {
                var ref = cordova.InAppBrowser.open(config.webServer + '/usr/about_point', '_blank', 'location=no');
                ref.addEventListener('loadstart', function () {
                    if (window.cordova && window.cordova.plugins.Keyboard) {
                        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
                    }
                });
                ref.addEventListener('exit', function () {
                    if (window.cordova && window.cordova.plugins.Keyboard) {
                        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                    }
                });
            } else {
                $rootScope.parameter = config.webServer + '/usr/about_point';
                $rootScope.title = 'ポイントについて';
                $state.go('about-point');
            }
        }

        // Check customer token on enter
        $scope.$on('$ionicView.enter', function (event) {
            if (event.targetScope === $scope) {
                apiRequesterService.checkToken($rootScope.Customer.customer_id, $rootScope.Auth.access_token);
            }
        });
    }
})();
