/*
 * PosRegisterCtrl
 *
 * responsible for pos registration pos info input
 * view : register1.html
 */
(function() {
    'use strict';

    angular
        .module('starter')
        .controller('PosRegisterCtrl', PosRegisterCtrl);

    function PosRegisterCtrl($scope, $log, $ionicScrollDelegate, $rootScope, $state, $ionicLoading, apiRequesterService, errorFactory, Flash, config, $ionicPlatform, $ionicPopup) {
      $scope.date = {
        years: [],
        months: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
        days: []
      }

      $rootScope.$on('$stateChangeStart', function() {
        $ionicScrollDelegate.scrollTop();
      })

      let disable_change_date = false;
      let leapYear = function (year) {
        return ((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0);
      }
      let daysInMonth = function  (month, year) {
        return new Date(year, month, 0).getDate();
      }
      let generateDays = function () {
        let up_to = 0;
        if (leapYear($rootScope.birth_date.birth_year) && ($rootScope.birth_date.birth_month == 2)) {
          up_to = 29;
        } else if ($rootScope.birth_date.birth_month == 2) {
          up_to = 28;
        } else {
          up_to = daysInMonth($rootScope.birth_date.birth_month, $rootScope.birth_date.birth_year);
        }
        for(let x=1;x<=up_to;x++) {
          $scope.date.days.push(x);
        }
      }
      $scope.changeYear = function () {
        $scope.date.months.length = 0;
        $scope.date.days.length = 0;
        disable_change_date = ($rootScope.birth_date.birth_year == ((new Date()).getFullYear()));
        if (disable_change_date) {
          let current_month = new Date().getMonth()+1;
          $rootScope.birth_date.birth_month = '--'
          $rootScope.birth_date.birth_day = '--'
          for(let x=1;x<=current_month;x++) {
            $scope.date.months.push(x);
          }
          let current_day = new Date().getDate();
          if (current_month == $rootScope.birth_date.birth_month) {
            for(let x=1;x<=current_day;x++) {
              $scope.date.days.push(x);
            }
          } else {
            generateDays();
          }
        } else  {
          for(let x=1;x<=12;x++) {
            $scope.date.months.push(x);
          }
          generateDays();
        }
      }

      $scope.changeDate = function () {
        $scope.date.days.length = 0;
        if (!disable_change_date) {
          generateDays();
          $rootScope.birth_date.birth_day = '--'
        } else {
          let current_month = new Date().getMonth()+1;
          let current_day = new Date().getDate();
          if (current_month == $rootScope.birth_date.birth_month) {
            for(let x=1;x<=current_day;x++) {
              $scope.date.days.push(x);
            }
            $rootScope.birth_date.birth_day = '--'
          } else {
            $rootScope.birth_date.birth_day = '--'
            generateDays();
          }
        }
      }
      // create a calendar
      let current_year = (new Date()).getFullYear();
      let old_year = (new Date()).getFullYear()-100;

      for(let x=current_year;x>=old_year;x--) {
        $scope.date.years.push(x);
      }

      $rootScope.birth_date = {
        birth_year: '----',
        birth_month: '--',
        birth_day: '--',
      }

      $scope.error = {
        name: false,
        birth_date: false,
        phone_no: false,
        postal: false,
        email: false,
        password: false,
        password_confirm: false,
        gender: false,
        mail: false,
        zip_code: false,
      }

      $rootScope.input = {
        name: '',
        birth_date: '',
        phone_no: '',
        email: '',
        password: '',
        password_confirm: '',
        gender: '1',
        mail: '0',
        zip_code1: '',
        zip_code2: '',
        zip_code: '',
      }

      $scope.error_messages = {
        name: '',
        birth_date: '',
        phone_no: '',
        email: '',
        password: '',
        password_confirm: '',
        gender: '',
        mail: '',
        zip_code: ''
      }

      $scope.returnBack = function () {
        $state.go('pos-terms');
      }

      var validateForm = function (params) {
        $ionicLoading.show(config.loading);
        let inputs = params;

        // clear error messages
        for(let input in inputs) {
          $scope.error[input] = false;
        }

        if ($rootScope.birth_date.birth_year
          && $rootScope.birth_date.birth_month
          && $rootScope.birth_date.birth_day
          && ($rootScope.birth_date.birth_year !== '----')
          && ($rootScope.birth_date.birth_month !== '--')
          && ($rootScope.birth_date.birth_day !== '--')) {
          $rootScope.input.birth_date
            = $rootScope.birth_date.birth_year + '-' + $rootScope.birth_date.birth_month + '-' + $rootScope.birth_date.birth_day;
        } else {
          $rootScope.input.birth_date = '';
        }

        apiRequesterService.validatePosRegister($rootScope.input).then(function(res){
          $ionicLoading.hide();
          $state.go('pos-register-confirm');
        }, function(err) {
          for(let i in err) {
            if (typeof err[i] === 'object') {
              for(let x in err[i]) {
                $scope.error[x] = true;
                $scope.error_messages[x] = err[i][x][0];
              }
            }
          }
          if (err.pos_customer_exist == 1) {
            var alertPopup = $ionicPopup.alert({
              title: 'エラー',
              template: '既に登録されている会員情報です。',
              cssClass: 'popup-customer-registration',
              buttons: [
                {
                  text: 'いいえ',
                  type: 'button-disable'
                },
                {
                  text: '会員認証<br>へ進む',
                  type: 'button-positive',
                  onTap: function(e) {
                    $state.go('confirm');
                  },
                }
              ]
            });
          }

          $ionicScrollDelegate.scrollTop();
          $ionicLoading.hide();
        })
      }

      $scope.clickValidate = function () {
        apiRequesterService.checkConnection().then(function(res){
          validateForm($rootScope.input);
        }, function(err) {
          $ionicLoading.hide()
          $rootScope.connected = 'noConnection';
        })
      }

      $scope.removeSpace = function () {
        $('input[type=text]')[0].value = $('input[type=text]')[0].value.replace(' ', '');
        $('input[type=text]')[0].value = $('input[type=text]')[0].value.replace('　', '');
      }

      var isAndroid = ionic.Platform.isAndroid();
      $scope.isAndroid = config.isAndroid;
      if ( config.isAndroid ) {
        $scope.classForAndroid = 'form--register-android';
      }

      $(document).ready(function() {

        $(".birthday").on('touchstart', function(e) {
          if (window.cordova && window.cordova.plugins.Keyboard && config.isIOS) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
          }
        });

        $(".birthday").on('blur', function(e) {
          if (window.cordova && window.cordova.plugins.Keyboard && config.isIOS) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
          }
        });

        if (isAndroid) {
          $('input[type=text]').keyup(function(e) {
            if( e.keyCode == 13 ) {
              validateForm($rootScope.input);
            }
          });
        }
      });
    }
})();
