/*
 * MyPageCtrl
 *
 * responsible for displaying customer points
 * view : mypage.html
 */
(function() {
  'use strict';
  angular
    .module('starter')
    .controller('MapCtrl', MapCtrl);

  function MapCtrl($scope, $state, $rootScope, $cordovaGeolocation, $ionicLoading, config, $stateParams, apiRequesterService, $ionicModal, $moment, $ionicPopup, $ionicScrollDelegate, $ionicPlatform, $cordovaAppAvailability, $sce) {

    var isIOS = ionic.Platform.isIOS();
    var isAndroid = ionic.Platform.isAndroid();
    $scope.searchVal;

    $ionicLoading.show(config.loading);
    $scope.lat         = null;
    $scope.lng         = null;
    $scope.km          = 15;
    $scope.shop_click  = null;
    $scope.showLocationButton = true;

    var shops_tmp        = [];
    var shops_marker_tmp = [];

    let day  = $moment().format('D');
    // if day is already greater than 10 get last month
    if (day > 10) {
      $scope.last_month = $moment().subtract('1',  'month').format('YYYY年MM月度');
    } else if (day == 10) { // check if time is > 6:10
       let time = $moment().format('H:mm').split(':');
       if (time[0] > 6) { // more than 6 am
          $scope.last_month = $moment().subtract('1',  'month').format('YYYY年MM月度');
       } else if (time[0] == 6 && time[1] >= 10) { // already 6 and minutes is greater = to 10 minutes
          $scope.last_month = $moment().subtract('1',  'month').format('YYYY年MM月度');
       } else {
          $scope.last_month = $moment().subtract('2',  'month').format('YYYY年MM月度');
       }
    } else {
      $scope.last_month = $moment().subtract('2',  'month').format('YYYY年MM月度');
    }

    $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/shop-detail-modal.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.modal = modal;
    });

    $scope.cms_host = config.imageServer;
    $scope.openWebView = function(shop_id, access_token) {
      var url = '/';
      if ($scope.Shop.url_type == 1) {
        url = config.webServer + '/usr/reservations/login_from_app?from_shop=' + shop_id;
        if (access_token != null) {
          url += '&token=' + access_token;
        }
      }
      if ($scope.Shop.url_type == 2) {
        url = $scope.Shop.url_for_booking_online;
      }
      var confirmPopup = $ionicPopup.confirm({
          title: 'ブラウザで開く',
          template: 'このページはアプリでは開けないため、ブラウザが起動しますがよろしいですか？',

          buttons: [{
              text: 'いいえ',
              type: 'button-disable'
          },{
              text: 'はい',
              type: 'button-positive',
              onTap: function(e) {
                  window.open(url, '_system', 'location=yes')
              }
          }
          ]
      });
    }

    $scope.openModal = function() {
      $ionicLoading.show(config.loading);
      $scope.Shop             = $scope.shop_click.data.Shop;
      $rootScope.connected = 'connected';
      let condition;
      condition = '?shop_id='+ $scope.Shop.shop_id;
      // Shop API
      apiRequesterService.getShops(condition).then(function(res) {
        $scope.Shop = res.data[0].Shop;
        $scope.ShopService      = null
        $scope.TherapistRanking = null
        // use noimage in cms if no image 1 and 2
        $scope.Shop.image_1 = $scope.Shop.image_1 != '' ? $scope.cms_host+'/images/shop/'+$scope.Shop.shop_id+'/'+$scope.Shop.image_1
                              : $scope.cms_host+'/img/noimage.png';
        $scope.Shop.image_2 = $scope.Shop.image_2 != '' ? $scope.cms_host+'/images/shop/'+$scope.Shop.shop_id+'/'+$scope.Shop.image_2
                              : $scope.cms_host+'/img/noimage.png';
        let url_encoded_address = encodeURIComponent($scope.Shop.address);
        $scope.Shop.map_location = "geo:?q="+url_encoded_address;
        if (isIOS) {
          $cordovaAppAvailability.check('comgooglemaps://').then(function () {
              $scope.Shop.map_location = 'comgooglemaps://?q='+ url_encoded_address;
          }, function () {
              $scope.Shop.map_location = 'http://maps.apple.com/?q='+ url_encoded_address;
          });
        }
        $scope.Shop.remarks = $sce.trustAsHtml($scope.Shop.remarks);
        // get shop services
        apiRequesterService.getShopServices($scope.Shop.shop_id).then(function(res) {
            let counter = 1;
            for (var i = 0; i < 4; i++) {
              res.data[i].ShopService.image = 'common/service_'+counter+'_off.png';
              if (res.data[i].ShopService.ShopServiceRelation.id != null) {
                res.data[i].ShopService.image = 'common/service_'+counter+'_on.png';
              }
              counter++;
            }
            $scope.ShopService = res.data;
        }, function() {
            $ionicLoading.hide();
        });
        // get therapist ranknings
        apiRequesterService.getTherapistRanksByShop($scope.Shop.shop_id).then(function(res) {
            let rank = 0;
            $scope.TherapistRanking = [];
            res.data.forEach(function(value, key){

              if (rank == 0 || parseInt(value.TherapistRank.appoint_sum) < parseInt(res.data[key - 1].TherapistRank.appoint_sum)) {
                rank++;
              }

              $scope.TherapistRanking.push({
                data: value,
                rank: rank
              });
            });

        }, function() {
            $ionicLoading.hide();
        });

        $ionicScrollDelegate.$getByHandle('modalContent').scrollTop(true);
        $ionicLoading.hide();
        $scope.modal.show();

      }, function() {
          $ionicLoading.hide();
      });
    }

    $scope.closeModal = function() {
      $scope.modal.hide();
    };
    // return first word from name
    $scope.mySplit = function(string, nb) {
      var array = string.split('　');
      return array[nb];
    }

    var options = {timeout: 10000, enableHighAccuracy: true};
    var mapOptions = {
        zoom: 11.5,
        mapTypeId: google.maps.MapTypeId.ROADMAP,
        disableDefaultUI: true,
    };
    $scope.map = new google.maps.Map(document.getElementById("map"), mapOptions);
    var pacContainer = document.getElementsByClassName('pac-container pac-logo hdpi');

    // Create the search box and link it to the UI element.
    var input = document.getElementById('pac-input');
    var searchBox = new google.maps.places.SearchBox(input);
    $scope.disableTap = function () {
        var container = document.getElementsByClassName('pac-container');
        angular.element(container).attr('data-tap-disabled', 'true');
        var backdrop = document.getElementsByClassName('backdrop');
        angular.element(backdrop).attr('data-tap-disabled', 'true');
        // leave input field if google-address-entry is selected
        angular.element(container).on("click", function () {
            document.getElementById('pac-input').blur();
        });
        $scope.handleSearchLocationClick();

        google.maps.event.addDomListener(input, "keydown", function(event) {
          if (isIOS && event.keyCode === 13) {
            input.value = $scope.searchVal;
            angular.element(pacContainer).css("display", "block");
            getLocationBySearch(input.value);
            cordova.plugins.Keyboard.close();
          }

          if (isAndroid && event.keyCode === 13) {
            input.value = $scope.searchVal;
            getLocationBySearch(input.value);
            cordova.plugins.Keyboard.close();
          }
        });

    };
    $scope.searchButton = function() {
        getLocationBySearch($scope.searchVal);
    }
    function getLocationBySearch(address) {
        var geocoder = new google.maps.Geocoder();
        geocoder.geocode({ 'address': address }, function (results, status) {
            if (status == google.maps.GeocoderStatus.OK) {
                var latitude = results[0].geometry.location.lat();
                var longitude = results[0].geometry.location.lng();
                $scope.is_requesting = false;
                $scope.map.setCenter(new google.maps.LatLng(latitude,longitude));
                $scope.map.setZoom(11.5);
            }
        });
    }

    // Bias the SearchBox results towards current map's viewport.
    $scope.map.addListener('bounds_changed', function() {
      searchBox.setBounds($scope.map.getBounds());
      angular.element(pacContainer).css("display", "none");
    });
    searchBox.addListener('places_changed', function() {
      $scope.searchVal = angular.element(input).val();
      getLocationBySearch($scope.searchVal);
    });
    $scope.is_requesting = false;
    $scope.show_shop_current = function(map) {
      if ($scope.is_requesting == false) {
        $scope.is_requesting = true
        if (typeof map.getCenter() == 'undefined') {
          return false;
        }
        $scope.lat = map.getCenter().lat();
        $scope.lng = map.getCenter().lng();
        // Zoom Level per km
        let bounds = map.getBounds();
        let center = bounds.getCenter();
        let ne = bounds.getNorthEast();
        // r = radius of the earth in statute km
        let r = 6371.0;
        // Convert lat or lng from decimal degrees into radians (divide by 57.2958)
        let lat1 = center.lat() / 57.2958;
        let lon1 = center.lng() / 57.2958;
        let lat2 = ne.lat() / 57.2958;
        let lon2 = ne.lng() / 57.2958;
        // distance = circle radius from center to Northeast corner of bounds
        $scope.km = r * Math.acos(Math.sin(lat1) * Math.sin(lat2) +
          Math.cos(lat1) * Math.cos(lat2) * Math.cos(lon2 - lon1));
        $scope.get_shops({lat:$scope.lat,lng: $scope.lng,km: $scope.km, is_latLng: true, change_zoom: false});
      }
    }

    //Listener for every trigger events in map like: click , zoom_changed ,dragend etc etc
    google.maps.event.addListener($scope.map, 'idle', function(){
       $scope.show_shop_current(this)
    });

    google.maps.event.addListener($scope.map, 'tilesloaded', function(){
       $scope.is_requesting = false;
    });

    function addMarkerListener(marker) {
      google.maps.event.addListener(marker, 'click', function() {
        $scope.shop_click = marker;
        $scope.openModal();
      });
    }

    $scope.get_shops = function(options) {
      let shop_is_id_only = false;
      let shop_id,lat,lng,km;
      if (options.is_latLng === true) {
        lat = options.lat;
        lng = options.lng;
        km  = options.km;
      } else {
        shop_is_id_only = true
        shop_id = options.shop_id;
      }

      $ionicLoading.show(config.loading);
      $rootScope.connected = 'connected';
      let condition;
      if (shop_is_id_only) {
        condition = '?shop_id='+ shop_id;
      } else {
        condition = '?latitude='+lat+'&longitude='+lng+'&kilometer='+km;
      }
      // Shop API
      apiRequesterService.getShops(condition).then(function(res) {
          var prev_shop = shops_tmp;
          let shop_vacancies_ids = [];

          angular.forEach(res.data, function (v, k) {
            if (typeof shops_tmp[v.Shop.shop_id] == 'undefined') {
              shops_tmp[v.Shop.shop_id] = v;
            } else {
              // remove marker
              if (typeof shops_marker_tmp[v.Shop.shop_id] != 'undefined') {
                shops_marker_tmp[v.Shop.shop_id].setMap(null);
              }
            }
            shop_vacancies_ids.push(v.Shop.shop_id);
          });

          // meaning no shop added in the array
          if (shop_vacancies_ids.length !== 0) {
            // return ids  = 1,2,3,4 if array is [1,2,3,4]
            let ids = shop_vacancies_ids.join();
            // Shop Vacancies API
            apiRequesterService.getVacancies(ids).then(function(res) {
              angular.forEach(res.data, function (v, k) {
                let shop_data = shops_tmp[v.PosShopVacancy.shop_no];
                let latLng = new google.maps.LatLng(shop_data.Shop.latitude, shop_data.Shop.longitude);
                if (typeof shop_data != 'undefined' ) {
                  let marker_color;
                  switch(v.PosShopVacancy.status) {
                      case '1':
                          marker_color = 'blue';
                          break;
                      case '2':
                          marker_color = 'red';
                          break;
                      case '3':
                          marker_color = 'grey';
                          break;
                      default:
                          marker_color = 'blue';
                  }

                  let time_split = v.PosShopVacancy.business_hour_start.split(':');
                  if (time_split[0] > 24) {
                    time_split[0] = time_split[0] - 24;
                    v.PosShopVacancy.business_hour_start = time_split[0]+':'+time_split[1];
                  }

                  let today_date = $moment().format("YYYY-MM-DD") + ' '+ v.PosShopVacancy.business_hour_start;
                  let pos_time = $moment(today_date);
                  let today_obj = $moment(new Date(res.headers('Date')));
                  let minutes_diff = pos_time.diff(today_obj, 'minutes');
                  // change marker color regardless of status if opening time is before 30 minutes
                  if (minutes_diff >= 0 && minutes_diff <= 30) { // 30 mins difference
                      marker_color = 'grey_in_preparing';
                  }

                  var icon = {
                    url: "img/marker_"+marker_color+".png", // url
                    scaledSize: new google.maps.Size(75, 100), // scaled size
                    origin: new google.maps.Point(0,0), // origin
                    // anchor: new google.maps.Point(0, 0) // anchor anchor: new google.maps.Point(0, 0) // anchor
                  };
                  // Marker Setup
                  var marker = new google.maps.Marker({
                      map: $scope.map,
                      position: latLng,
                      icon: icon,
                      data: shop_data,
                  });
                  shops_marker_tmp[v.PosShopVacancy.shop_no] = marker;
                  // add listener for every marker that created
                  addMarkerListener(marker);
                  if (options.change_zoom) {
                    $scope.map.setZoom(11.5);
                  }
                  if (shop_is_id_only) {
                    // set focus on the shop
                    $scope.map.setCenter(new google.maps.LatLng(shop_data.Shop.latitude, shop_data.Shop.longitude), 15);
                    $scope.map.setZoom(15);
                  }
                  $scope.is_requesting = true;
                }
              });
            });
          }
          $ionicLoading.hide();
      }, function() {
          $ionicLoading.hide();
      });
    }
    var infoWindow = new google.maps.InfoWindow;
    function handleLocationError(browserHasGeolocation, infoWindow, pos) {
      if (pos == null) {
        return false;
      }
      infoWindow.setPosition(pos);
      infoWindow.setContent(browserHasGeolocation ?
                            'Error: The Geolocation service failed.' :
                            'Error: Your browser doesn\'t support geolocation.');
      infoWindow.open(map);
    }

    // show current posiition
    $scope.show_current = function() {
      // cordova.plugins.locationManager.getAuthorizationStatus().then(function(authorized){
      //   if (authorized.authorizationStatus == 'AuthorizationStatusDenied') {
      //       //...code
      //   }
      // }).catch(function(error){
      //   console.log(error);
      // });
      if (navigator.geolocation) {
          navigator.geolocation.getCurrentPosition(function(position) {
            var pos = {
              lat: position.coords.latitude,
              lng: position.coords.longitude
            };
            let latLng = new google.maps.LatLng(pos.lat, pos.lng);
            $scope.map.setCenter(new google.maps.LatLng( pos.lat, pos.lng ) );
            var userMarker = new google.maps.Marker({
              position: latLng,
              anchor: new google.maps.Point(50, 50),
              url: "https://openclipart.org/download/82549/blue-circle.svg",
              map: $scope.map,
            });
          }, function() {
            let latLng = new google.maps.LatLng('34.67', '135.46'); // Osaka
            $scope.map.setCenter(new google.maps.LatLng( '34.67', '135.46') );
          });
        } else {
          // Browser doesn't support Geolocation
          handleLocationError(false, infoWindow, $scope.map.getCenter());
          // re-call the function to show native pop up
          navigator.geolocation.getCurrentPosition(function(position) {

          }), function(error) {
            console.log(error);
          };
        }
        $ionicLoading.hide();
    }
    $scope.show_current();

    $scope.$on('$stateChangeSuccess', function(event, toState, toParams, fromState) {
      if ($state.current.url == '/shops/map' && !['pref', 'city', 'list'].includes(fromState.name)) {
        $rootScope.connected = 'connected';
        apiRequesterService.checkVacancies().then(function(res) {
          if (res.data.PosShopVacancy.status == 'false' || res.data.PosShopVacancy.status == false && fromState.name == 'login' && $rootScope.reservation_login == 1) {
            var alertPopup = $ionicPopup.alert({
              title: 'システムメンテナンス中',
              template: '地図上に表示されている店舗の空き情報が正しくない可能性がございますので、店舗にお電話してご確認いただくようお願いいたします。',
              buttons: [{text: '閉じる', type: 'button--cancel'}]
            });
          }
        });
      }
      
      if ($state.current.url == '/shops/map' && typeof $rootScope.ShopId != 'undefined' ) {
          $ionicLoading.show(config.loading);
          $scope.get_shops({shop_id:$rootScope.ShopId , is_latLng: false, change_zoom: true})
          if (fromState.name == 'stamp-rally-shop_list') {
            $rootScope.showBackButton = true;
            $('.header__menu').toggleClass('header__menu--back');
          }
      } else {
        $rootScope.ShopId = undefined
      }

      if (fromState.name == 'login' && $rootScope.reservation_login == 1) {
        $scope.modal.show();
      }

      $rootScope.reservation_login = null;
    });

    $scope.handleSearchLocationClick = function () {
      $scope.showLocationButton = !$scope.showLocationButton;
    }

    $scope.handleInputChange = function() {
      $scope.searchVal = angular.element(input).val();
    }

    $scope.clickMapLocation = function() {
      window.location.href = $scope.Shop.map_location;
    }

    $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/web-reservation-modal.html', {
      scope: $scope,
      animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.modal1 = modal;
    });

    $scope.openReservationModal = function(shop_id) {
      $scope.shop_id = shop_id;

      if ($scope.Shop.url_type == 1) {
        if (typeof $rootScope.Customer === 'undefined' || $rootScope.Customer === null) {
          $scope.modal1.show();
        } else {
          $scope.openWebView(shop_id, $rootScope.Auth.access_token);
        }
      }
      if ($scope.Shop.url_type == 2) {
        $scope.openWebView(shop_id, null);
      }
    };

    $scope.closeReservationModal = function() {
      $scope.modal1.hide();
    };

    $scope.goToLogin = function() {
      $rootScope.from_reservation = 1;
      $state.go('login');
      $scope.modal.hide();
    };

    $(document).ready(function() {
      $(document).delegate('#shop_remarks a', 'click', function(e) {
          e.preventDefault();
          $rootScope.parameter = $(this).attr('href');
          var target = '_system';
          if (config.isAndroid) {
            target = '_self'
          }
          var confirmPopup = $ionicPopup.confirm({
            title: 'ブラウザで開く',
            template: 'このページはアプリでは開けないため、ブラウザが起動しますがよろしいですか？',

            buttons: [{
                text: 'いいえ',
                type: 'button-disable'
            },{
                text: 'はい',
                type: 'button-positive',
                onTap: function(e) {
                    window.open($rootScope.parameter, target, 'location=yes')
                }
              }
            ]
          });
      });
  });
  }
})();
