/*
 * StampInfoCtrl
 *
 * responsible for stamp rally information
 * view : info.html
 */
(function() {
    'use strict';

    angular
        .module('starter')
        .controller('StampInfoCtrl', StampInfoCtrl);

    function StampInfoCtrl($scope, $log, $rootScope, $state,$ionicModal, $ionicLoading, apiRequesterService, errorFactory, Flash, $moment, $ionicPopup, config) {

      $ionicModal.fromTemplateUrl('order-history.html', {
        scope: $scope,
        animation: 'slide-in-up'
      }).then(function(modal) {
        $scope.modal = modal;
      });

      $scope.openOrderHistoryModal = function(index) {
        $scope.modal.show();
        $scope.order_detail_items = $scope.acquired_stamps[index];
        let getElement = angular.element( document.querySelector( '.modal' ) );
        $('.modal .scroll').css('transform','translate3d(0px, 0px, 0px) scale(1)');
        if( $scope.order_detail_items.length < 2 ) {
          getElement.addClass('modal--stamp-info-order');
        } else {
          getElement.removeClass('modal--stamp-info-order');
        }
      }

      $scope.openWebView = function(acquired_stamp) {
        var access_token = $rootScope.Auth.access_token;
        var url = '/';
          if (acquired_stamp.url_type == 1) {
            url = config.webServer + '/usr/reservations/login_from_app?from_shop=' + acquired_stamp.shop_id;
            if (access_token != null) {
              url += '&token=' + access_token;
            }
          }
          if (acquired_stamp.url_type == 2) {
            url = acquired_stamp.url_for_booking_online;
          }
        var confirmPopup = $ionicPopup.confirm({
            title: 'ブラウザで開く',
            template: 'このページはアプリでは開けないため、ブラウザが起動しますがよろしいですか？',

            buttons: [{
                text: 'いいえ',
                type: 'button-disable'
            },{
                text: 'はい',
                type: 'button-positive',
                onTap: function(e) {
                    window.open(url, '_system', 'location=yes')
                }
            }
            ]
        });
      }

      $scope.closeModal = function() {
        $scope.modal.hide();
      };

      // return first word from name
      $scope.mySplit = function(string, nb) {
          var array = string.split('　');
          return array[nb];
      }

      $scope.addReview = function(e) {
          $rootScope.order_select = {order_id:e.order_id,order_detail_id:e.id}
          $state.go('add');
          $scope.modal.hide();
      }

      $scope.acquired_stamps = [];
      $scope.order_detail_items = [];
      $scope.one_month_ago = $moment().subtract(1, 'Month').unix();
      var counter = 0;
      $scope.remaining_days = null;
      $scope.remaining_date = null;

      function setAcquiredStamp() {
        angular.forEach($rootScope.StampRally.PosAcquiredStamp, function(stamp_value, stamp_key){
          $scope.acquired_stamps[stamp_key] = [];
          angular.forEach(stamp_value.PosOrder.PosOrderDetail, function(order_detail_value, order_detail_key){
            $scope.acquired_stamps[stamp_key][counter] = {
              id: order_detail_value.id,
              order_id: stamp_value.PosOrder.id,
              registered_business_date: stamp_value.PosOrder.registered_business_date,
              registered_business_date_unix: $moment(stamp_value.PosOrder.registered_business_date).unix(),
              product_names : order_detail_value.product_names,
              total_sub_total : order_detail_value.total_sub_total,
              shop_name: stamp_value.PosOrder.shop_name,
              shop_id: stamp_value.PosOrder.PosMstShop.no,
              shop_telephone_no: stamp_value.PosOrder.PosMstShop.telephone_no,
              is_available_to_book_online: stamp_value.PosOrder.PosMstShop.is_available_to_book_online,
              url_type: stamp_value.PosOrder.PosMstShop.url_type,
              url_for_booking_online: stamp_value.PosOrder.PosMstShop.url_for_booking_online_sp,
              therapist_name: $scope.mySplit(order_detail_value.PosTherapistNomination[0].therapist_name, 0),
              stars: typeof order_detail_value.Review != 'undefined' ?  order_detail_value.Review.rank : null
            };
            counter++;
          });
          counter = 0;
        });
      }

      function getStampRallyFromAPI() {
        apiRequesterService.getStampRally($rootScope.Customer.customer_id, $rootScope.Auth).then(function(res) {
          if (res.data.length != 0) {
            $rootScope.StampRally = res.data;
            $scope.image_path = config.posImageServer + '/images/stamp_rally_images/';
            $scope.mst_stamp_rally = $rootScope.StampRally.PosMstStampRally;
            $scope.acquired_characters = $rootScope.StampRally.PosCustomerStampRally.acquired_stamp_string.split("");
            $scope.acquired_stamps_count = $rootScope.StampRally.PosCustomerStampRally.number_of_visits;
            $scope.status = $rootScope.StampRally.PosCustomerStampRally.status;

            let range = [];
            for(var i = 0; i < $scope.acquired_characters.length; i++) {
              range.push(i);
            }
            $scope.range = range;

            setAcquiredStamp();

            let max_execution_time = 60;
            apiRequesterService.getExpirationOfStampRally().then(function(response){
              max_execution_time = response.data.max_execution_time;

              // compute remaining days
              let created = $rootScope.StampRally.PosCustomerStampRally.created;
              // adapt new Date() to iOS format
              created = created.replace(/-/g, '/');
              created = created.slice( 0, -3 );
              let start_date = new Date(created);

              if (parseInt($moment(start_date).format('Hmm')) < 600) {
                start_date = $moment(start_date).subtract(1, 'Day');
              }
              start_date = $moment(start_date).format('YYYY-MM-DD');

              let today = $moment();
              if (parseInt($moment().format('Hmm')) < 600) {
                today = $moment().subtract(1, 'Day');
              }
              today = $moment(today).format('YYYY-MM-DD');

              let expiration_days = $moment(start_date).add(max_execution_time + 1, 'Day').format('YYYY-MM-DD');
              let expiration_date = $moment(start_date).add(max_execution_time, 'Day').format('YYYY-MM-DD');

              $scope.remaining_days = $moment(expiration_days).diff($moment(today), 'days');
              $scope.remaining_date = expiration_date;

            }, function(err) {
              $rootScope.StampRally = null;
              $state.go('home');
            });
          } else {
            $rootScope.StampRally = null;
            $state.go('home');
          }
        }, function(err) {
          $rootScope.StampRally = null;
          $state.go('home');
        });
      }

      $scope.$on('$stateChangeSuccess', function(event, toState) {
        if (toState.name == 'stamp-rally') {
          getStampRallyFromAPI();
        }
      });
    }
})();
