/*
 * TherapistRankListCtrl
 *
 * responsible for displaying order history email screen
 * view : list.html
 */
(function() {
    'use strict';

    angular
        .module('starter')
        .controller('TherapistRankListCtrl', TherapistRankListCtrl);

    function TherapistRankListCtrl($rootScope, $scope,config, $state, $timeout, $ionicModal, $ionicLoading, apiRequesterService, $moment, $ionicScrollDelegate, $ionicPopup, $cordovaAppAvailability) {
        $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/shop-detail-modal.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
        });

        let day  = $moment().format('D');
        // if day is already greater than 10 get last month
        if (day > 10) {
            $scope.last_month = $moment().subtract('1',  'month').format('YYYY年MM月度');
        } else if (day == 10) { // check if time is > 6:10
            let time = $moment().format('H:mm').split(':');
            if (time[0] > 6) { // more than 6 am
                $scope.last_month = $moment().subtract('1',  'month').format('YYYY年MM月度');
            } else if (time[0] == 6 && time[1] >= 10) { // already 6 and minutes is greater = to 10 minutes
                $scope.last_month = $moment().subtract('1',  'month').format('YYYY年MM月度');
            } else {
                $scope.last_month = $moment().subtract('2',  'month').format('YYYY年MM月度');
            }
        } else {
            $scope.last_month = $moment().subtract('2',  'month').format('YYYY年MM月度');
        }

        $scope.closeModal = function() {
            $scope.modal.hide();
        };
        $scope.$on('modal.hidden', function() {
            $ionicScrollDelegate.$getByHandle('modalContent').scrollTop(true);
        });

        $scope.openModal = function(event) {
            $ionicLoading.show(config.loading);
            $scope.Shop             = event.rank_item;
            $scope.ShopService      = null
            $scope.TherapistRanking = null
            $scope.cms_host         = config.imageServer;
            
            let condition = '?shop_id='+$scope.Shop.shop_id;
            apiRequesterService.getShops(condition).then(function(res) {
                $rootScope.connected = 'connected';
                $scope.Shop = res.data[0].Shop;
                $scope.Shop.image_1 = $scope.Shop.image_1 != '' ? $scope.cms_host+'/images/shop/'+$scope.Shop.shop_id+'/'+$scope.Shop.image_1: $scope.cms_host+'/img/noimage.png';
                $scope.Shop.image_2 = $scope.Shop.image_2 != '' ? $scope.cms_host+'/images/shop/'+$scope.Shop.shop_id+'/'+$scope.Shop.image_2: $scope.cms_host+'/img/noimage.png';
                let url_encoded_address = encodeURIComponent($scope.Shop.address);
                $scope.Shop.map_location = "geo:?q="+url_encoded_address;
                if (ionic.Platform.isIOS()) {
                    $cordovaAppAvailability.check('comgooglemaps://').then(function () {
                        $scope.Shop.map_location = 'comgooglemaps://?q='+ url_encoded_address;
                    }, function () {
                        $scope.Shop.map_location = 'http://maps.apple.com/?q='+ url_encoded_address;
                    });
                }

                // get shop services
                apiRequesterService.getShopServices($scope.Shop.shop_id).then(function(res) {
                    let counter = 1;
                    for (var i = 0; i < 4; i++) {
                        res.data[i].ShopService.image = 'common/service_'+counter+'_off.png';
                        if (res.data[i].ShopService.ShopServiceRelation.id != null) {
                            res.data[i].ShopService.image = 'common/service_'+counter+'_on.png';
                        }
                        counter++;
                    }
                    $scope.ShopService = res.data;
                }, function(err) {
                    $rootScope.connected = 'noConnection';
                });
                // get therapist ranknings
                apiRequesterService.getTherapistRanksByShop($scope.Shop.shop_id).then(function(res) {
                    let rank = 0;
                    $scope.TherapistRanking = [];
                    res.data.forEach(function(value, key){

                      if (rank == 0 || parseInt(value.TherapistRank.appoint_sum) < parseInt(res.data[key - 1].TherapistRank.appoint_sum)) {
                        rank++;
                      }

                      $scope.TherapistRanking.push({
                        data: value,
                        rank: rank
                      });
                    });
                    $scope.modal.show();
                    $ionicLoading.hide();
                }, function(err) {
                    $rootScope.connected = 'noConnection';
                });
            }, function(err) {
                $rootScope.connected = 'noConnection';
            });

            $ionicLoading.hide();
            $ionicScrollDelegate.$getByHandle('modalContent').scrollTop(true);
        }

        $scope.openWebView = function(shop_id, access_token) {
          var url = '/';
          if ($scope.Shop.url_type == 1) {
            url = config.webServer + '/usr/reservations/login_from_app?from_shop=' + shop_id;
            if (access_token != null) {
              url += '&token=' + access_token;
            }
          }
          if ($scope.Shop.url_type == 2) {
            url = $scope.Shop.url_for_booking_online;
          }
          var confirmPopup = $ionicPopup.confirm({
              title: 'ブラウザで開く',
              template: 'このページはアプリでは開けないため、ブラウザが起動しますがよろしいですか？',

              buttons: [{
                  text: 'いいえ',
                  type: 'button-disable'
              },{
                  text: 'はい',
                  type: 'button-positive',
                  onTap: function(e) {
                      window.open(url, '_system', 'location=yes')
                  }
              }
              ]
          });
        }

        $scope.pref_id = '';
        $scope.pref_name = '全国';

        $scope.refreshing = false;
        $rootScope.rank_limit = 5;
        $rootScope.noMoreItemsAvailable = false;
        $scope.counter = 0;

        __displayTherapistRanks($rootScope.rank_limit);

        // Populate Therapist Ranks
        function __displayTherapistRanks(new_limit, options) {
            let global_limit = $rootScope.rank_limit;
            if (options == null) {
              global_limit = $rootScope.rank_limit;
            }
            apiRequesterService.getTherapistRanks($scope.pref_id, global_limit).then(function(res) {

                if (res.data.length != 0 && (res.data.length > $scope.counter || global_limit == 5) ){
                    $scope.items = [];

                    var display_rank;
                    var old_asum = null, new_asum = null, rank_asum = null;
                    $rootScope.rank_limit = 5;
                    $scope.counter = res.data.length

                    angular.forEach(res.data, function(value, key) {
                        // Rank by therapist_ranks.appoint_sum
                        old_asum = value.TherapistRank.appoint_sum;
                        if (old_asum !== new_asum) {
                            rank_asum++;
                        }
                        new_asum = old_asum;

                        // Assign image to rank if rank_asum 1-3
                        if (rank_asum == 1 || rank_asum == 2 || rank_asum == 3) {
                            display_rank = 'img/therapist-ranking/rank'+ rank_asum +'.png';
                        } else {
                            display_rank =  rank_asum + '位';
                        }

                        if (value.TherapistRank.professional_name == null || value.TherapistRank.professional_name == '') {
                            $scope.therapist_name = $scope.mySplit(value.TherapistRank.name,0);
                        } else {
                            $scope.therapist_name = $scope.mySplit(value.TherapistRank.professional_name,0);
                        }
                        $scope.items.push({
                            id: rank_asum,
                            rank_item: {
                                number: display_rank,
                                name : $scope.therapist_name,
                                pref: value.TherapistRank.Shop.pref_name,
                                shop_id : value.TherapistRank.Shop.shop_no,
                                place: value.TherapistRank.Shop.shop_name,
                                gender: value.TherapistRank.sex_type
                            }
                        });

                        if (res.data.length >= $rootScope.rank_limit) {
                            $rootScope.rank_limit+=5;
                        }

                    });
                    $scope.$broadcast('scroll.refreshComplete');

                    $rootScope.noMoreItemsAvailable = false;
                } else {
                    if (res.data.length == 0) {
                        $scope.items = [];
                    } else {
                        $scope.items;
                    }
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    $rootScope.noMoreItemsAvailable = true;
                }
            },function(err){
                $rootScope.noMoreItemsAvailable = true;
            }).finally(function() {
                // Stop the loading from spinning
                $scope.$broadcast('scroll.infiniteScrollComplete');
                $scope.$broadcast('scroll.refreshComplete');
                $timeout(function(){
                    $scope.refreshing = false;
                },1500);
            });
        }

        $scope.$on('$locationChangeSuccess',function(evt, absNewUrl, absOldUrl) {
            if (absOldUrl.split('#')[1] == '/therapist-ranks/pref') {
                $rootScope.rank_limit = 5;
                $rootScope.noMoreItemsAvailable = false;

                if (typeof $rootScope.selectedPrefId == 'undefined') {
                    $scope.pref_id = '';
                    $scope.pref_name = '全国';
                } else {
                    $scope.pref_id = $rootScope.selectedPrefId;
                    $scope.pref_name = $rootScope.selectedPrefName;
                }

                __displayTherapistRanks($rootScope.rank_limit);
            }
        });

        $scope.$on('$stateChangeSuccess', function(event, toState, toParams, fromState) {
            if (fromState.name == 'login' && $rootScope.reservation_login == 1) {
                $scope.modal.show();
            }

            $rootScope.reservation_login = null;
        });

        // return first word from name
        $scope.mySplit = function(string, nb) {
            var array = string.split('　');
            return array[nb];
        }

        $scope.doRefresh = function() {
            // Reset values
            $scope.pref_id = '';
            $scope.pref_name = '全国';

            $scope.refreshing = false;
            $rootScope.rank_limit = 5;
            $rootScope.noMoreItemsAvailable = false;
            $scope.counter = 0;

            __displayTherapistRanks($rootScope.rank_limit);
        }

        $scope.loadMore = function() {
            if (!$scope.noMoreItemsAvailable) {
                __displayTherapistRanks($rootScope.rank_limit, true);
            }
        }

        $scope.clickMapLocation = function() {
            window.location.href = $scope.Shop.map_location;
        }

        $ionicModal.fromTemplateUrl('js/ctrls/layouts/common/web-reservation-modal.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal1 = modal;
        });

        $scope.openReservationModal = function(shop_id) {
          $scope.shop_id = shop_id;

          if ($scope.Shop.url_type == 1) {
            if (typeof $rootScope.Customer === 'undefined' || $rootScope.Customer === null) {
              $scope.modal1.show();
            } else {
              $scope.openWebView(shop_id, $rootScope.Auth.access_token);
            }
          }
          if ($scope.Shop.url_type == 2) {
            $scope.openWebView(shop_id, null);
          }
        };

        $scope.closeReservationModal = function() {
          $scope.modal1.hide();
        };

        $scope.goToLogin = function() {
          $rootScope.from_reservation = 1;
          $state.go('login');
          $scope.modal.hide();
        };
    }
})();
