angular.module(
    'dbModule',
    []
).factory(
    'DbFactory',
    function($q) {

        return {
            /* method for creating local database table and inserting data
            * @param tableName name of the table that will be created
            * @param dataObject array of json object i.e. [{id:"1","name":"Alex"},{"id":"2","name":"Sven"}]
            * @return boolean true if success else false
            */
            insert: function(tableName, dataObject) {
                // get the object keys
                var keys = Object.keys(dataObject[0]),
                // implode the keys
                fields = keys.join(),
                q = $q.defer();
                db.transaction(function (tx) {
                    // create the table if not existing
                    tx.executeSql('CREATE TABLE IF NOT EXISTS ' + tableName + ' (' + fields + ')', [], function() {
                        // iterate through json object
                        for(i in dataObject) {
                            var values = [];
                            var qMark = [];
                            for (var key in dataObject[i]) {
                                // push the value per row
                                values.push(dataObject[i][key]);
                                qMark.push('?');
                            }
                            // create the query
                            var qMark_imp = qMark.join();
                            var sql = 'INSERT INTO ' + tableName + ' (' + fields + ') ' + 'VALUES (' + qMark_imp + ')';
                            // insert the data in the sql
                            tx.executeSql(sql, values);
                        }
                        q.resolve(true);
                    },
                    function() {
                        q.reject(false);
                    });
                });
                return q.promise;
            },

            /* method selecting data in the specific table
            * @param [String] tableName name of the table that will be selected
            * @values [Array] values array of values base on query
            * @return json data of the selected table
            */
            get: function(tableName, query, values) {
                var q = $q.defer(),
                result = [],
                sql = query != undefined ? query : '',
                v = values != undefined ? values : [];
                db.transaction(function (tx) {
                    tx.executeSql('SELECT * FROM ' + tableName + sql, v , function (tx, rs) {
                        // convert it to json object
                        for (var i=0; i < rs.rows.length; i++) {
                            result.push(rs.rows.item(i));
                        }
                        // resolve the result data
                        q.resolve(result);
                    },
                    function(transaction,err) {
                        q.reject(err.message);
                    });
                });
                // return the data
                return q.promise;
            },

            /* method for dropping table
            * @param tableName name of the table that will be dropped
            * @return boolean true if success else false
            */
            drop: function(tableName) {
                var q = $q.defer();
                db.transaction(function (tx) {
                    tx.executeSql('DROP TABLE IF EXISTS ' + tableName);
                    q.resolve(true);
                },
                function(tx, rs) {
                    q.reject(false);
                });
                return q.promise;
            },

            /* method for checking if table is existing
            * @param tableName name of the table that will be check
            * @return boolean true if existing else false
            */
            tableExist: function(tableName) {
                var q = $q.defer();
                db.transaction(function (tx) {
                    tx.executeSql('SELECT 1 FROM ' + tableName + ' LIMIT 1', [], function() {
                        q.resolve(true);
                    },
                    function() {
                        q.reject(false);
                    });
                });
                return q.promise;
            }
        };
    }
);