angular.module(
    'apiRequesterModule',
    []
).service(
    'apiRequesterService',
    function($q, $http, config, $base64, $log, $rootScope) {

    var returnError = function( err ) {
        if ( err == null ) {
            return undefined;
        } else {
            return err.ErrorDetail;
        }
    }

    return {
        /**
        * registerUser for registration of customer
        * @param  json data email, password, and password confirm
        * @return promise
        */
        register: function(data) {
            var email = data.email ? data.email : '',
            password = data.password ? data.password : '',
            password_confirm = data.password_confirm ? data.password_confirm : '',
            mail_flg = data.mail_flg ? data.mail_flg : '',
            customer_id = data.customer_id ? data.customer_id : '',
            pref_id = data.pref_id ? data.pref_id : '',
            city = data.city ? data.city : '',
            street = data.street ? data.street : '',
            phone_no = data.phone_no ? data.phone_no : '';

            var post_data = { 'Customer':
                {
                    customer_id : customer_id,
                    email: email,
                    password: password,
                    password_confirm: password_confirm,
                    customer_id: customer_id,
                    phone_no: phone_no,
                    pref_id: pref_id,
                    city: city,
                    street: street,
                    mail_flg:mail_flg
                },
                is_new_api: 1
            };

            var q = $q.defer();
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/customers.json',
                data: post_data,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });

            return q.promise;
        },
        /**
        * login responsible for login function
        * @param  json data email, password
        * @return promise
        */
        login: function(data) {
            var q = $q.defer();
            var email = data.email ? data.email : '',
            password = data.password ? data.password : '';
            post_data = {'Customer':{email:email, password:password}};
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/customers/login.json',
                data: post_data,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });

            return q.promise;
        },

        /**
        * forgotPassword responsible for sending generate key
        * @param  string email customer email
        * @return promise
        */
        forgotPassword: function(email) {
            var email = email ? email : '';
            var q = $q.defer();
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/customers/reissue_password.json',
                data: {'Customer':{email:email, reset_password_device:2}},
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });

            return q.promise;
        },

        /**
        * getInfo get customer object base on id and token
        * @param  int id customer id
        * @param  string access_token access token
        * @return promise
        */
        getInfo: function(id, access_token) {
            var id = id ? id : 0,
            access_token = access_token ? access_token : 0;
            var q = $q.defer();
            var req = {
                method: 'GET',
                url: config.server + '/api/v2/customers.json?customer_id=' + id,
                timeout: config.timeout,
                headers: {
                    'X-Riracle-Token' : access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });

            return q.promise;
        },

        /**
        * logout function for logging out
        * @param  json auth access_token and expiry
        * @param  json customer id and email
        * @return promise
        */
        logout: function(auth, customer) {
            var auth = auth ? auth : 0,
            customer = customer ? customer : 0;
            var post_data = {};
            post_data.Customer = {};
            post_data.Customer.customer_id = customer.customer_id;
            var q = $q.defer();
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/customers/logout.json',
                data: post_data,
                headers: {
                    'X-Riracle-Token' : auth.access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },

        /**
        * updateEmail responsible for updating email
        * @param  json auth auth object
        * @param  json customer customer object
        * @param  string email new email address
        * @return promise
        */
        updateEmail: function(auth, customer, email) {
            var default_email = customer.email;
            var auth = auth ? auth : 0,
            customer = customer ? customer : 0,
            post_data = {};
            post_data.Customer = {};
            post_data.Customer.page = 'change_mail';
            post_data.Customer.email = email;
            post_data.Customer.customer_id = customer.customer_id;
            var q = $q.defer();
            var req = {
                method: 'PUT',
                url: config.server + '/api/v2/customers.json',
                data: post_data,
                timeout: config.timeout,
                headers: {
                    'X-Riracle-Token' : auth.access_token
                }
            }

            $http(req).then(function(res){
                customer.email = default_email;
                q.resolve(true);
            },
            function(res){
                customer.email = default_email;
                q.reject(returnError(res.data));
            });

            return q.promise;
        },
        /**
        * updateNoti responsible for updating notification
        * @param  json auth auth object
        * @param  json customer customer object
        * @return promise
        */
        updateNoti: function(auth, customer) {
            var customer_id = customer.customer_id;
            push_noti = customer.push_noti ? 1 : 0 ;
            post_data = {};
            post_data.Customer = {};
            post_data.Customer.customer_id = customer_id;
            post_data.Customer.push_noti = push_noti;
            post_data.Customer.page = 'change_push';
            var q = $q.defer();
            var req = {
                method: 'PUT',
                url: config.server + '/api/v2/customers.json',
                data: post_data,
                headers : {
                    'X-Riracle-Token' : auth.access_token
                },
                timeout: config.timeout,
            }

            $http(req).then(function(res){
                q.resolve(true);
            },
            function(res){
                q.reject(returnError(res.data));
            });

            return q.promise;
        },

        /**
        * updateEmailFlg responsible for updating emailflg
        * @param  json auth auth object
        * @param  json customer customer object
        * @return promise
        */
        updateEmailFlg: function(auth, customer) {
            var customer_id = customer.customer_id;
            mailflg = customer.mailflg ? 1 : 0;
            post_data = {};
            post_data.Customer = {};
            post_data.Customer.customer_id = customer_id;
            post_data.Customer.mailflg = mailflg;
            post_data.Customer.page = 'change_mailflg'
            var q = $q.defer();
            var req = {
                method: 'PUT',
                url: config.server + '/api/v2/customers.json',
                data: post_data,
                headers : {
                    'X-Riracle-Token' : auth.access_token
                },
                timeout: config.timeout,
            }

            $http(req).then(function(res){
                q.resolve(true);
            },
            function(res){
                q.reject(returnError(res.data));
            });

            return q.promise;
        },

        /**
        * deactivate responsible for deactivating account
        * @param  json auth auth object
        * @param  json customer customer object
        * @return promise
        */
        deactivate: function(auth, customer) {
            var auth = auth ? auth : 0,
            customer = customer ? customer : 0,
            post_data = {};
            post_data.Customer = customer;
            var q = $q.defer();
            var req = {
                method: 'DELETE',
                url: config.server + '/api/v2/customers.json?customer_id='+post_data.Customer.customer_id,
                headers : {
                    'X-Riracle-Token' : auth.access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },

        /**
        * changePassword responsible for changing customer password
        * @param  json auth auth object
        * @param  json customer customer object
        * @return promise
        */
        changePassword: function(auth, customer) {
            var auth = auth ? auth : 0,
            customer = customer ? customer : 0,
            post_data = {};
            post_data.Customer = {};
            post_data.Customer.customer_id = customer.customer_id ;
            post_data.Customer.password = customer.password ;
            post_data.Customer.password_confirm = customer.password_confirm ;
            post_data.Customer.page = 'change_password';
            var q = $q.defer();
            var req = {
                method: 'PUT',
                url: config.server + '/api/v2/customers.json',
                data: post_data,
                headers: {
                    'X-Riracle-Token' : auth.access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(true);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * changeTel responsible for changing customer password
        * @param  json auth auth object
        * @param  json customer customer object
        * @return promise
        */
        changeTel: function(auth, customer) {
            var auth = auth ? auth : 0,
            customer = customer ? customer : 0,
            post_data = {};
            post_data.Customer = {};
            post_data.Customer.customer_id =  customer.customer_id ;
            post_data.Customer.phone_no =  customer.phone_no ;
            post_data.Customer.page =  'change_tel';
            var q = $q.defer();
            var req = {
                method: 'PUT',
                url: config.server + '/api/v2/customers.json',
                data: post_data,
                timeout: config.timeout,
                headers : {
                    'X-Riracle-Token' : auth.access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(true);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },

        /**
        * confirmation responsible for checking customer code and tel
        * @param  int cc  customer code
        * @param  int tel tel number
        * @return promise
        */
        confirmation: function(cc, tel) {
            var cc = cc ? cc : '',
            tel = tel ? tel : '';

            var q = $q.defer();
            var req = {
                method: 'GET',
                url: config.server + '/api/v2/customers/auth.json?members_card_no=' + cc + '&phone_no=' + tel,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },

        /**
        * getOrders responsible for getting orders
        * @param  int customer_id customer id
        * @param  string access_token customer access_token
        * @param  int limit order list limit
        * @param  int offset order offset
        * @param  string review
        * @param  string order_id order id
        * @return promise
        */
        getOrders: function(customer_id, auth, limit, offset, review,order_id) {
            var customer_id = customer_id ? customer_id : 0,
            access_token = auth.access_token ? auth.access_token : 0;
            review = review == null ? '' : '&review=on';
            order_id = order_id == null ? '' : '&order_id='+order_id;

            var q = $q.defer();
            var url = config.server + '/api/v2/orders.json?customer_id=' + customer_id + '&limit=' + limit + '&offset=' + offset+review+order_id;
            var req = {
                url: url,
                method: 'GET',
                headers : {
                    'X-Riracle-Token' :  access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * addReviews responsible for adding or update review
        * @param int customer_id customer id
        * @param string access_token
        * @param array data
        * @param bool is_update
        * @return promise
        */
        addReviews: function(customer_id, auth, data, is_update) {
            var customer_id = customer_id ? customer_id : 0,
            access_token = auth.access_token ? auth.access_token : 0;
            var post_data = {};
            var method = is_update == null ? 'POST' : 'PUT';
            post_data.Review = {
                'customer_id': customer_id,
                'order_detail_id': data.order_detail == null ? '' : data.order_detail ,
                'comment': data.comment == null ? '' : data.comment ,
                'rank': data.rank == null ? '' : data.rank
            };

            var q = $q.defer();
            var url = config.server + '/api/v2/reviews.json';
            var req = {
                method: method,
                url: url,
                data:post_data,
                headers : {
                    'X-Riracle-Token' :  access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * confirmCode responsible for confirming code
        * @param  string email customer email
        * @param  int reset_password_key for resetting password
        * @return promise
        */
        confirmCode: function(email, reset_password_key) {

            var email = email ? email : 0,
            reset_password_key = reset_password_key ? reset_password_key : "",
            post_data = {'Customer':{'email':email, 'reset_password_key':reset_password_key}};
            var q = $q.defer();
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/customers/code_auth.json',
                data: post_data,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },

        /**
        * newPassword responsible for creating new password
        * @param  json data data includes email, password, and confirm password
        * @return promise
        */
        newPassword: function(data) {
            var customer_id = data.customer_id ? data.customer_id : '';
            var password = data.password ? data.password : '';
            var password_confirm = data.password_confirm ? data.password_confirm : '';

            post_data = {'Customer':{'customer_id':customer_id, 'password':password, 'password_confirm':password_confirm}};
            var q = $q.defer();
            var req = {
                method: 'PUT',
                url: config.server + '/api/v2/customers/new_passwd.json',
                data: post_data,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(true);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },

        registerDeviceId: function(customer_id, device_id, access_token) {
            var post_data = {'Customer':{'customer_id':customer_id, 'device_id':device_id}};
            var q = $q.defer();
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/fcm_customers.json',
                data: post_data,
                headers: {
                    'X-Riracle-Token' : access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(true);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * checkConnection responsible for checking internet connection
        * @return promise
        */
        checkConnection: function() {
            var q = $q.defer();
            var req = {
                method: 'POST',
                url: config.server,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(true);
            },
            function(res){
                q.reject(true);
            });

            return q.promise;
        },
        /**
        * regenerateAccessToken responsible for regenerating access token
        * @param  json auth auth object
        * @param  json customer customer object
        * @return promise
        */
        regenerateAccessToken: function(auth, customer) {
            var q = $q.defer();
            var post_data = {};
            post_data.Customer = {};
            post_data.Customer.customer_id = customer.customer_id;
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/auth_customers.json',
                data: post_data,
                timeout: config.timeout,
                headers : {
                    'X-Riracle-Token'        : auth.access_token,
                    'X-Riracle-RefreshToken' : auth.refresh_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getPrefectures responsible for getting all prefectures
        * @return promise
        */
        getPrefectures: function() {
            var q = $q.defer();
            var req = {
                method: 'GET',
                url: config.server + '/api/v2/prefs.json',
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getCities responsible for getting all cities
        * @param  querystring of pref_id
        * @return promise
        */
        getCities: function(pref) {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/cities.json?pref_id='+pref.id ,
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getShops responsible for getting all shops
        * @param  querystring of shopid
        * @return promise
        */
        getShops: function(condition) {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/shops.json'+condition ,
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getVacancies responsible for getting all vacancies
        * @param  querystring of shopids
        * @return promise
        */
        getVacancies: function(shop_ids) {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/shop_vacancies.json?shop_id='+shop_ids,
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * checkVacancies responsible for checking if JBShopVacancyShell is not working properly
        * @return promise
        */
        checkVacancies: function() {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/shop_vacancies/check.json',
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getShopServices responsible for getting all shop services
        * @param  querystring of shopids
        * @return promise
        */
        getShopServices: function(shop_ids) {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/services.json?shop_id='+shop_ids,
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * changeAddress responsible for changing address
        * @param  json auth auth object
        * @param  json customer customer object
        * @return promise
        */
        changeAddress : function(auth, customer) {
            var auth = auth ? auth : 0,
            customer = customer ? customer : 0,
            post_data = {};
            post_data.Customer = {};
            post_data.Customer.customer_id =  customer.customer_id;
            post_data.Customer.street = customer.street ? customer.street : '';
            post_data.Customer.city =  customer.city ? customer.city : '';
            post_data.Customer.pref_id = customer.pref_id ;
            post_data.Customer.page =  'change_address';
            var q = $q.defer();
            var req = {
                method: 'PUT',
                url: config.server + '/api/v2/customers.json',
                data: post_data,
                timeout: config.timeout,
                headers : {
                    'X-Riracle-Token' : auth.access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * registerSurvey responsible for registration of survey
        * @param  json customer customer object
        * @param  json auth auth object
        * @param  json survey_answer survey_answer object
        * @return promise
        */
        selectRewardPoints: function(type) {
            var q = $q.defer();

            var req = {
                method: 'GET',
                url: config.server + '/api/v2/reward_points.json?target_type='+type,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getSurveys responsible for getting all questions and survey info
        * @return promise
        */
        getSurveys: function() {
            var q = $q.defer();

            var req = {
                method: 'GET',
                url: config.server + '/api/v2/surveys.json',
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * registerSurvey responsible for registration of survey
        * @param  json customer customer object
        * @param  json auth auth object
        * @param  json survey_answer survey_answer object
        * @return promise
        */
        registerSurvey: function(customer,auth,survey_answer) {
            var q = $q.defer();
            var post_data = survey_answer;
            post_data.AuthCustomer = auth;
            post_data.Customer = customer;
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/surveys.json',
                data: post_data,
                headers : {
                    'X-Riracle-Token' : auth.access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;

        },
        /**
        * selectRewardPoints responsible for registration of survey
        * @param type  int target_type int
        * @return promise
        */
        selectRewardPoints: function(type) {
            var q = $q.defer();

            var req = {
                method: 'GET',
                url: config.server + '/api/v2/reward_points.json?target_type='+type,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * registerPoint responsible for registration of give point
        * @param  json customer customer object
        * @param  json auth auth object
        * @param  json answer_id int
        * @param  json answer_type int
        * @return promise
        */
        registerPoint: function(customer,auth,answer_id,answer_type, points) {
            var q = $q.defer();
            var post_data               = {}
            post_data.Point             = {};
            post_data.Point.customer_id = customer.customer_id;
            post_data.Point.point       = points;
            post_data.Point.target_id   = answer_id;
            post_data.Point.target_type = answer_type;
            var req = {
                method: 'POST',
                url: config.server + '/api/v2/points.json',
                data: post_data,
                headers : {
                    'X-Riracle-Token' : auth.access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getBanners responsible for getting banners
        * @param  string type type of banner
        * @param  int limit banner list limit
        * @param  int offset banner offset
        * @return promise
        */
        getBanners: function(type, limit, offset) {
            limit = limit == null ? 0 : limit;
            offset = offset == null ? 0 : offset;
            type = type == null ? null : type;
            var q = $q.defer();
            var url = config.server + '/api/v2/banners.json?limit=' + limit + '&offset=' + offset + '&type=' + type;
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getNews responsible for getting news
        * @param  int news_type news type
        * @param  int limit news list limit
        * @param  int offset news offset
        * @param  int news_id news id
        * @return promise
        */
        getNews: function(news_type, limit, offset, news_id) {
            limit = limit == null ? 5 : limit;
            offset = offset == null ? 0 : offset;
            news_type = news_type == null ? null : news_type;
            news_id = news_id == null ? null : news_id;
            var q = $q.defer();
            var url = config.server + '/api/v2/news.json?limit=' + limit + '&offset=' + offset;
            if (news_type != null) {
                url += '&news_type=' + news_type;
            }
            if (news_id != null) {
                url += '&id=' + news_id;
            }
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getShops responsible for getting all shops
        * @param  querystring of shopid
        * @return promise
        */
        getShops: function(condition) {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/shops.json'+condition ,
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getShopServices responsible for getting all shop services
        * @param  querystring of shopids
        * @return promise
        */
        getShopServices: function(shop_ids) {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/services.json?shop_id='+shop_ids,
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getTherapistRankings responsible for getting all therapist rankings
        * @param  querystring of shopids
        * @return promise
        */
        getTherapistRankings: function(shop_ids) {
            var q = $q.defer();
            var req = {
                method : 'GET',
                url : config.server + '/api/v2/therapist_ranks.json?shop_no='+shop_ids,
                timeout: config.timeout
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getTherapistRanks responsible for getting therapist ranks
        * @param  pref_id
        * @return promise
        */
        getTherapistRanks: function(pref_id, limit) {
            var q = $q.defer();
            var req = {
                method: 'GET',
                url: config.server + '/api/v2/therapist_ranks.json?pref_id=' + pref_id + '&limit=' + limit,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getTherapistRanksByShop responsible for getting therapist ranks
        * @param  shop_id
        * @return promise
        */
        getTherapistRanksByShop: function(shop_id, limit) {
            var q = $q.defer();
            var req = {
                method: 'GET',
                url: config.server + '/api/v2/therapist_ranks.json?shop_no=' + shop_id + '&type=detail',
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * selectPoint get customer object base on id and token
        * @param  int id customer id
        * @param  string access_token access token
        * @return promise
        */
        selectPoint: function(id, access_token, limit, offset) {
            limit = limit == null ? 10 : limit;
            offset = offset == null ? 0 : offset;
            var id = id ? id : 0,
            access_token = access_token ? access_token : 0;
            var q = $q.defer();
            var req = {
                method: 'GET',
                url: config.server + '/api/v2/points.json?customer_id=' + id+'&limit='+limit+'&offset='+offset,
                headers : {
                    'X-Riracle-Token' : access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });

            return q.promise;
        },
        /**
        * getCoupons responsible for getting coupons
        * @param  int customer_id customer id
        * @return promise
        */
        getCoupons: function(customer_id, access_token) {
            var param = '';
            var req = {
                method: 'GET',
                timeout: config.timeout,
            }
            if (customer_id != null && access_token != null) {
                param = '?customer_id=' + customer_id;
                req.headers = {'X-Riracle-Token' : access_token};
            }
            var q = $q.defer();
            var url = config.server + '/api/v2/coupons.json' + param;
            req.url = url;
            $http(req).then(function(res){
                $rootScope.coupon_count = res.data.length;
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },/**
        * getCouponTerms responsible for getting coupon terms
        * @return promise
        */
        getCouponTerms: function() {
            var q = $q.defer();
            var url = config.server + '/api/v2/coupon_terms.json';
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getAnswers responsible for getting answers
        * @param  int customer_id int survey_id int, access_token string,
        * @return promise
        */
        getAnswers: function(survey_id,customer_id,access_token) {
            customer_id = customer_id == null ? null : customer_id;
            survey_id   = survey_id == null ? null : survey_id;
            var q = $q.defer();
            access_token = access_token ? access_token : 0;
            var url = config.server + '/api/v2/answers.json?customer_id=' + customer_id+'&survey_id='+survey_id
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
                headers: {
                    'X-Riracle-Token' : access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * setLastLoginApp responsible for getting answers
        * @param  int customer_id int access_token string,
        * @return promise
        */
        setLastLoginApp: function(customer_id,access_token) {
            var date = new Date();
            var year = date.getFullYear();
            var month = date.getMonth() + 1;
            var day = date.getDate();
            var hours = date.getHours();
            var minutes = date.getMinutes();
            var seconds = date.getSeconds();
            var last_login =  year + "-" + month + "-" + day + " " + hours + ":" + minutes + ":" + seconds;

            var q = $q.defer();
            var url = config.server + '/api/v2/customers.json';
            var post_data = {
                Customer : {
                    customer_id    : customer_id,
                    page           : 'last_login_app',
                    last_login_app : last_login
                }
            }
            var req = {
                method: 'PUT',
                url: url,
                data: post_data,
                headers : {
                    'X-Riracle-Token' : access_token
                },
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * checkToken responsible for checking token if its alive or expired
        * @param  int customer_id int  access_token string,
        * @return promise
        */
        checkToken: function(customer_id,access_token) {
            customer_id = customer_id == null ? null : customer_id;
            var q = $q.defer();
            access_token = access_token ? access_token : 0;
            var url = config.server + '/api/v2/customers/check_token.json?customer_id=' + customer_id;
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
                headers: {
                    'X-Riracle-Token' : access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },

        /**
        * getStampRally responsible for getting stamp rally
        * @param  int customer_id, access_token string
        * @return promise
        */
        getStampRally: function(customer_id, auth) {
            customer_id = customer_id == null ? null : customer_id;
            var q = $q.defer();
            access_token = auth.access_token ? auth.access_token : 0;
            var url = config.server + '/api/v2/stamp_rallies.json?customer_id=' + customer_id;
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
                headers: {
                    'X-Riracle-Token' : access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getStampRallyShops responsible for getting stamp rally shops
        * @param  int mst_stamp_rally_id
        * @return promise
        */
        getStampRallyShops: function(mst_stamp_rally_id, shop_ids, limit, offset) {
            limit = limit == null ? 0 : limit;
            offset = offset == null ? 0 : offset;
            mst_stamp_rally_id = mst_stamp_rally_id == null ? null : mst_stamp_rally_id;
            var q = $q.defer();
            var url = config.server + '/api/v2/stamp_rally_shops.json?mst_stamp_rally_id=' + mst_stamp_rally_id + '&shop_ids=' + shop_ids + '&limit=' + limit + '&offset=' + offset;
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getPreviousStampRally responsible for getting latest expired stamp rally
        * @param  int customer_id, access_token string
        * @return promise
        */
        getPreviousStampRally: function(customer_id, auth) {
            customer_id = customer_id == null ? null : customer_id;
            var q = $q.defer();
            access_token = auth.access_token ? auth.access_token : 0;
            var url = config.server + '/api/v2/stamp_rallies/previous.json?customer_id=' + customer_id;
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
                headers: {
                    'X-Riracle-Token' : access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getExpirationOfStampRally responsible for getting expiration of stamp rally
        * @return promise
        */
        getExpirationOfStampRally: function() {
            var q = $q.defer();
            var url = config.server + '/api/v2/stamp_rallies/expiration.json';
            var req = {
                method: 'GET',
                url: url,
                timeout: config.timeout,
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        /**
        * getTermsOfServices responsible for getting terms of service
        * @return json
        */
        getTermsOfServices: function() {
          var q = $q.defer();
          var url = config.server + '/api/v2/terms.json';
          var req = {
              method: 'GET',
              url: url,
              timeout: config.timeout,
          }
          $http(req).then(function(res){
              q.resolve(res);
          },
          function(res){
              q.reject(returnError(res.data));
          });
          return q.promise;
        },
         /**
        * validatePosRegister for validating of customer registration form
        * @param json Customer { email, password, password_confirm
        * password_confirm, mail_flg } PosCustomer { genderd, phone_no,
        * zip_code1, birth_date, zip_code2 }
        * @return empty if success, json of error messages
        */
       validatePosRegister: function (params) {
        let q = $q.defer();
        let url = config.server + '/api/v2/pos_customers/validate.json';
        let hp_customer = {
          Customer: {
            email:            params.email || '',
            password:         params.password || '',
            password_confirm: params.password_confirm || '',
            mailflg:         params.mail || ''
          }
        }

        let pos_customer = {
          PosCustomer: {
            name:       params.name || '',
            gender:     params.gender || '',
            phone_no:   params.phone_no || '',
            zip_code1:  params.zip_code1 || '',
            birth_date: params.birth_date || '',
            zip_code2:  params.zip_code2 || '',
          }
        }

        let post_data = Object.assign(hp_customer, pos_customer);
        var req = {
            method: 'POST',
            url: url,
            data: post_data,
        }
        $http(req).then(function(res){
            q.resolve(res);
        },
        function(res){
            q.reject(returnError(res.data));
        });

        return q.promise;
       },
        /**
        * posRegister creates a new customer record in hp and pos table
        * @param json Customer { email, password, password_confirm
        * password_confirm, mail_flg } PosCustomer { genderd, phone_no,
        * zip_code1, birth_date, zip_code2 }
        * @return empty if success, json of error messages
        */
       posRegister: function (params) {
        let param_mail = 0
        if (params.mail === '0') {
          param_mail = 1;
        }
        let q = $q.defer()
        let url = config.server + '/api/v2/pos_customers.json';
        let hp_customer = {
          Customer: {
            email:            params.email || '',
            password:         params.password || '',
            password_confirm: params.password_confirm || '',
            mailflg:          param_mail
          }
        }

        let pos_customer = {
          PosCustomer: {
            name:       params.name || '',
            gender:     params.gender || '',
            phone_no:   params.phone_no || '',
            zip_code1:  params.zip_code1 !==''? params.zip_code1 : null,
            birth_date: params.birth_date || '',
            zip_code2:  params.zip_code2 !==''? params.zip_code2 : null,
          }
        }

        let post_data = Object.assign(hp_customer, pos_customer);
        var req = {
            method: 'POST',
            url: url,
            data: post_data,
        }
        $http(req).then(function(res){
            q.resolve(res);
        },
        function(res){
            q.reject(returnError(res.data));
        });

        return q.promise;
       },
        /**
        * posCustomerActivate activates a none activated customer record
        * @param json Customer id and activation key
        * @return empty if success, json of error messages
        */
       posCustomerActivate: function (params) {
        let q = $q.defer()
        let url = config.server + '/api/v2/pos_customers/activate.json'
        let post_data = {
          Customer: {
            id: params.id || '',
            activation_key: params.activation_key || '',
          }
        }
        var req = {
            method: 'POST',
            url: url,
            data: post_data,
        }
        $http(req).then(function(res){
            q.resolve(res);
        },
        function(res){
            q.reject(returnError(res.data));
        });
        return q.promise;
       },
       /**
        * posCustomerChangeEmail handles customer email update
        * @param json Customer id and email
        * @return empty if success, json of error messages
        */
       posCustomerChangeEmail: function (params) {
        let q = $q.defer()
        let url = config.server + '/api/v2/customers/change_email.json'
        let post_data = {
          Customer: {
            id: params.id || '',
            email: params.email || '',
          }
        }
        var req = {
            method: 'POST',
            url: url,
            data: post_data,
        }
        $http(req).then(function(res){
            q.resolve(res);
        },
        function(res){
            q.reject(returnError(res.data));
        });

        return q.promise;
       },
       /**
        * getCustomerHome responsible for getting customer home page data and setting last login
        * @param  int customer_id, access_token string
        * @return promise
        */
        getCustomerHome: function(customer_id, access_token) {
            customer_id = customer_id == null ? null : customer_id;
            access_token = access_token ? access_token : 0;
            var date = new Date();
            var year = date.getFullYear();
            var month = date.getMonth() + 1;
            var day = date.getDate();
            var hours = date.getHours();
            var minutes = date.getMinutes();
            var seconds = date.getSeconds();
            var last_login =  year + "-" + month + "-" + day + " " + hours + ":" + minutes + ":" + seconds;

            var post_data = {
                Customer : {
                    customer_id    : customer_id,
                    last_login_app : last_login
                }
            }

            var q = $q.defer();
            var url = config.server + '/api/v2/customers/home.json';
            var req = {
                method: 'POST',
                url: url,
                data: post_data,
                timeout: config.timeout,
                headers: {
                    'X-Riracle-Token' : access_token
                }
            }
            $http(req).then(function(res){
                q.resolve(res);
            },
            function(res){
                q.reject(returnError(res.data));
            });
            return q.promise;
        },
        checkForUpdate: function (platform) {
            var q = $q.defer();
            var url = config.server + '/api/v2/app_updates/check.json';
            var req = {
                method: 'GET',
                url: url,
                params: {
                    platform: platform
                },
                timeout: config.timeout
            }

            $http(req).then(
                function (res) {
                    q.resolve(res);
                },
                function (res) {
                    q.reject(returnError(res.data));
                }
            );
            return q.promise;
        },
        requestSmsVerification: function (id, phone_no, validation_only) {
            var q = $q.defer();
            var url = config.server + '/api/v2/customers/issue_sms_code.json';
            var req = {
                method: 'POST',
                url: url,
                data: {
                    Customer: {
                        id: id,
                        phone_no: phone_no,
                        validation_only: validation_only
                    }
                },
                timeout: config.timeout
            }

            $http(req).then(
                function (res) {
                    q.resolve(res);
                },
                function (res) {
                    q.reject(returnError(res.data));
                }
            );

            return q.promise;
        },
        confirmSmsVerification: function (id, activation_sms_key) {
            var q = $q.defer();
            var url = config.server + '/api/v2/customers/verify_phone_key.json';
            var req = {
                method: 'POST',
                url: url,
                data: {
                    Customer: {
                        id: id,
                        activation_sms_key: activation_sms_key || null
                    }
                },
                timeout: config.timeout
            }

            $http(req).then(
                function (res) {
                    q.resolve(res);
                },
                function (res) {
                    q.reject(returnError(res.data));
                }
            );

            return q.promise;
        }
    } /* end return */
}/* end function($q, $http, config, $base64, $log) */
);
